package sim.application.util;


import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;

import sim.exception.SConstructorException;
import sim.exception.SNoImplementationException;
import sim.graphics.SBufferedImageUtil;
import sim.nn.layer.SFullConnectedLayer;
import sim.nn.network.SFullConnectedNetwork;
import sim.nn.network.SNetwork;
import sim.nn.reader.SNeuralImageDataReader;
import sim.nn.trainer.STrainer;
import sim.util.SLog;

import java.awt.GridLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JTextField;
import javax.swing.JLabel;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.Color;

import javax.swing.JScrollPane;
import javax.swing.JTextPane;

/**
 * La classe <b>SJFrameColorClassifier</b> reprsente une application permettant d'effectuer
 * l'apprentissage de la classification des pixels de couleur dans une image.
 * 
 * @author Simon Vezina
 * @since 2020-03-18
 * @version 2022-09-13
 */
public class SJFrameColorClassifier extends JFrame {

  /**
   * ...
   */
  private final String DEFAULT_ACTIVATION_FUNCTION_NAME = "sigmoid";
  
  /**
   * ...
   */
  private final String DEFAULT_IMAGE_FILE_NAME = "quatre_points.png";
  
  /**
   * 
   */
  private static final long serialVersionUID = 7983188152677065937L;
  
  /**
   * ...
   */
  private final String DEFAULT_ERROR_FUNCTION = "energy";
  
  /**
   * La variable <b>reader</b> reprsente le lecteur d'image de couleurs  classifier.
   */
  SNeuralImageDataReader reader = null;
   
  /**
   * La variable <b>trainer</b> reprsente l'entraneur de rseau (contenant le rseau).
   */
  private STrainer trainer = null;
    
  private JPanel contentPane;
  private JTextField textFieldLearningRate;
  private JTextField textFieldBatchSize;

  private JTextField textFieldNbTraining;
  private JLabel labelErrorNetworkValue;
  private JLabel labelTestingClassificationValue;
 
  private SJPanelImage panelImage;
  private SJPanelImage panelTrainingImage;
  private SJPanelImage panelTestingImage;
  private SJPanelImage panelErrorClassificationImage;
  private JLabel labelNbTraining;
  private JLabel labelNbTrainingVector;
  
  private JTextPane textPaneNetworkDescription;
  private JTextField textFieldImageFileName;
  private JTextField textFieldNetworkDimension;
  
  /**
   * ...
   * 
   * @param network
   * @param reader
   * @param trainer
   */
  public SJFrameColorClassifier(STrainer trainer, SNeuralImageDataReader reader) 
  {
    this();
    
    this.trainer = trainer;
    this.reader = reader;
    
    //------------------
    // INITIALISATION //
    //------------------
    this.trainer.addTraining(this.reader.getAllTrainingData());
    this.trainer.addTesting(this.reader.getAllTestingData()); 
    
    //-------------------------------------------
    // INITIALISATION DE L'INTERFACE GRAPHIQUE //
    //-------------------------------------------
    this.textPaneNetworkDescription.setText(this.trainer.getNetwork().getDescription());
    
    this.textFieldImageFileName.setText(this.reader.getFileName());
    
    setImage();
    setLabel();
  }
  
  /**
   * Create the frame.
   */
  public SJFrameColorClassifier()
  {
    setTitle("Classification de couleur");
    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        
    setBounds(50, 50, 1200, 800);
    contentPane = new JPanel();
    contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
    setContentPane(contentPane);
    contentPane.setLayout(new BorderLayout(0, 0));
    
    JPanel panelCenter = new JPanel();
    contentPane.add(panelCenter, BorderLayout.CENTER);
    panelCenter.setLayout(new GridLayout(0, 4, 0, 0));
    
    JPanel panel_1 = new JPanel();
    panelCenter.add(panel_1);
    panel_1.setLayout(new BorderLayout(0, 0));
    
    this.panelImage = new SJPanelImage();
    panel_1.add(panelImage, BorderLayout.CENTER);
    //panelImage.setMinimumSize(new Dimension(200, 200));
    this.panelImage.setPreferredSize(new Dimension(200, 200));
    
    JLabel lblNewLabel = new JLabel("Image de classification");
    panel_1.add(lblNewLabel, BorderLayout.NORTH);
    
    JPanel panel_2 = new JPanel();
    panelCenter.add(panel_2);
    panel_2.setLayout(new BorderLayout(0, 0));
    
    this.panelTrainingImage = new SJPanelImage();
    panel_2.add(panelTrainingImage, BorderLayout.CENTER);
    //panelTrainingImage.setMinimumSize(new Dimension(200, 200));
    this.panelTrainingImage.setPreferredSize(new Dimension(200, 200));
    
    JLabel lblNewLabel_1 = new JLabel("Donn\u00E9es d'entra\u00EEnement");
    panel_2.add(lblNewLabel_1, BorderLayout.NORTH);
    
    JPanel panel_3 = new JPanel();
    panelCenter.add(panel_3);
    panel_3.setLayout(new BorderLayout(0, 0));
    
    this.panelTestingImage = new SJPanelImage();
    panel_3.add(panelTestingImage, BorderLayout.CENTER);
    //panelTestingImage.setMinimumSize(new Dimension(200, 200));
    this.panelTestingImage.setPreferredSize(new Dimension(200, 200));
    
    JLabel lblNewLabel_2 = new JLabel("Donn\u00E9es de test");
    panel_3.add(lblNewLabel_2, BorderLayout.NORTH);
    
    JPanel panel_4 = new JPanel();
    panelCenter.add(panel_4);
    panel_4.setLayout(new BorderLayout(0, 0));
    
    this.panelErrorClassificationImage = new SJPanelImage();
    panel_4.add(panelErrorClassificationImage, BorderLayout.CENTER);
    //panelErrorClassificationImage.setMinimumSize(new Dimension(200, 200));
    this.panelErrorClassificationImage.setPreferredSize(new Dimension(200, 200));
    
    JLabel lblNewLabel_3 = new JLabel("Classification en erreur");
    panel_4.add(lblNewLabel_3, BorderLayout.NORTH);
    
    JPanel panelSouth = new JPanel();
    contentPane.add(panelSouth, BorderLayout.SOUTH);
    panelSouth.setLayout(new GridLayout(2, 4, 0, 0));
    
    JPanel panel5 = new JPanel();
    panelSouth.add(panel5);
    
    JButton buttonTrain = new JButton("Train");
    
    buttonTrain.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) 
      {
        train();
      }
    });
    
    JButton buttonClassify = new JButton("Classifier");
    buttonClassify.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        classify();
      }
    });
    panel5.add(buttonClassify);
    panel5.add(buttonTrain);
    
    JPanel panel6 = new JPanel();
    panelSouth.add(panel6);
    
    JLabel labelLearningRate = new JLabel("Learning rate :");
    panel6.add(labelLearningRate);
    
    textFieldLearningRate = new JTextField();
    textFieldLearningRate.setText("0.1");
    panel6.add(textFieldLearningRate);
    textFieldLearningRate.setColumns(10);
    
    JPanel panel7 = new JPanel();
    panelSouth.add(panel7);
    
    JLabel labelBatchSize = new JLabel("Batch size :");
    panel7.add(labelBatchSize);
    
    textFieldBatchSize = new JTextField();
    textFieldBatchSize.setText("1");
    panel7.add(textFieldBatchSize);
    textFieldBatchSize.setColumns(10);
    
      JPanel panel8 = new JPanel();
      panelSouth.add(panel8);
      
      JLabel labelNbTrain = new JLabel("Nb training :");
      panel8.add(labelNbTrain);
      
      textFieldNbTraining = new JTextField();
      textFieldNbTraining.setText("100");
      panel8.add(textFieldNbTraining);
      textFieldNbTraining.setColumns(10);
      
      JPanel panel9 = new JPanel();
      panelSouth.add(panel9);
      
      JLabel labelErrorNetwork = new JLabel("Erreur du r\u00E9seau :");
      panel9.add(labelErrorNetwork);
      
      labelErrorNetworkValue = new JLabel("");
      panel9.add(labelErrorNetworkValue);
      
      JPanel panel10 = new JPanel();
      panelSouth.add(panel10);
      
      JLabel labelTestingClassification = new JLabel("Taux de classification : ");
      panel10.add(labelTestingClassification);
      
      labelTestingClassificationValue = new JLabel("");
      panel10.add(labelTestingClassificationValue);
      
      JPanel panel11 = new JPanel();
      panelSouth.add(panel11);
      
      JLabel lblNewLabel_4 = new JLabel("Nombre d'entra\u00EEnement :");
      panel11.add(lblNewLabel_4);
      
      labelNbTraining = new JLabel("");
      panel11.add(labelNbTraining);
      
      JPanel panel12 = new JPanel();
      panelSouth.add(panel12);
      
      JLabel lblNewLabel_5 = new JLabel("Nb vecteur utilis\u00E9s :");
      panel12.add(lblNewLabel_5);
      
      labelNbTrainingVector = new JLabel("");
      panel12.add(labelNbTrainingVector);
      
      JPanel panelNorth = new JPanel();
      contentPane.add(panelNorth, BorderLayout.NORTH);
      panelNorth.setLayout(new GridLayout(1, 2, 0, 0));
      
      JPanel panel = new JPanel();
      panelNorth.add(panel);
      panel.setLayout(new GridLayout(2, 1, 0, 0));
      
      JPanel panel_6 = new JPanel();
      panel.add(panel_6);
      
      JLabel lblNewLabel_6 = new JLabel("Fichier image :");
      panel_6.add(lblNewLabel_6);
      
      textFieldImageFileName = new JTextField();
      panel_6.add(textFieldImageFileName);
      textFieldImageFileName.setText(DEFAULT_IMAGE_FILE_NAME);
      textFieldImageFileName.setColumns(20);
      
      JButton buttonImageSelection = new JButton("S\u00E9lection image");
      panel_6.add(buttonImageSelection);
      
      JButton btnBrowser = new JButton("Browser ...");
      btnBrowser.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          actionBrowser();
        }
      });
      panel_6.add(btnBrowser);
      buttonImageSelection.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          imageSelection();
        }
      });
      
      JPanel panel_5 = new JPanel();
      panel.add(panel_5);
      
      JLabel lblNewLabel_8 = new JLabel("Dimension du r\u00E9seau :");
      panel_5.add(lblNewLabel_8);
      
      textFieldNetworkDimension = new JTextField();
      panel_5.add(textFieldNetworkDimension);
      textFieldNetworkDimension.setColumns(20);
      
      JButton buttonNetworkSelection = new JButton("S\u00E9lection du r\u00E9seau");
      buttonNetworkSelection.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          networkSelection();
        }
      });
      panel_5.add(buttonNetworkSelection);
      
      JPanel panel_7 = new JPanel();
      panelNorth.add(panel_7);
      panel_7.setLayout(new BorderLayout(0, 0));
      
      JLabel lblNewLabel_7 = new JLabel("Description du r\u00E9seau");
      panel_7.add(lblNewLabel_7, BorderLayout.NORTH);
      
      this.textPaneNetworkDescription = new JTextPane();
      textPaneNetworkDescription.setMinimumSize(new Dimension(400, 200));
      textPaneNetworkDescription.setMaximumSize(new Dimension(400, 200));
      this.textPaneNetworkDescription.setPreferredSize(new Dimension(400, 200));
      panel_7.add(this.textPaneNetworkDescription);
      
      JScrollPane scrollPane = new JScrollPane(this.textPaneNetworkDescription, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      scrollPane.setMaximumSize(new Dimension(400, 200));
      panel_7.add(scrollPane);
      
      // INITIALISATION DES IMAGES ET LABEL.
      setImage();
      setLabel();
  }

  /**
   * Mthode pour raliser un cycle d'entranement du rseau.
   */
  private void train()
  {
    try {
      
      // Obtenir les donnes de l'interface graphique.
      double learning_rate = Double.parseDouble(this.textFieldLearningRate.getText());
      int batch_size = Integer.parseInt(this.textFieldBatchSize.getText());
      int nb_training = Integer.parseInt(this.textFieldNbTraining.getText());
      
      this.trainer.setLearningRate(learning_rate);
      
      // Faire l'entranement.
      for(int i = 0; i < nb_training; i++)
        this.trainer.train(batch_size);
      
      // valuer l'erreur sur le rseau.
      this.trainer.computeErrorsTestingData();
      
      // Affichage des labels.
      setLabel();
      
      // Changement des images 
      setImage();           
      
    }catch(NumberFormatException e) {
      SLog.logException(e);
    }catch(NullPointerException e) {
      SLog.logWriteLine("L'entranement n'est pas possible, car le rseau n'a pas t dfini.");
    }catch(SNoImplementationException e) {
      SLog.logWriteLine("L'entranement n'est pas possible, car une mthode ncessaire au fonctionnement n'a pas t implmente.");
    }
  }
  
  /**
   * Action permettant de dfinir le nom du fichier en lecture.
   */
  private void actionBrowser()
  {
    JFileChooser jfc = new JFileChooser(System.getProperty("user.dir"));

    int returnValue = jfc.showOpenDialog(null);
    
    if (returnValue == JFileChooser.APPROVE_OPTION) 
    {
      File selectedFile = jfc.getSelectedFile();
      this.textFieldImageFileName.setText(selectedFile.getName());  
    }
  }
  
  private void classify()
  {
    try {
      
      this.trainer.computeErrorsTestingData();
      
      setLabel();
      
      // Changement de l'image des erreurs de classification.
      this.panelErrorClassificationImage.setBufferedImage(reader.getClassificationErrorTestingImage());
               
    }catch(NullPointerException e) {
      SLog.logWriteLine("La classification n'est pas possible, car le rseau n'a pas t dfini.");
    }catch(SNoImplementationException e) {
      SLog.logWriteLine("La classification n'est pas possible, car une mthode ncessaire au fonctionnement n'a pas t implmente.");
    }
  }
  
  /**
   * Mthode pour dfinir le contenu des labels de l'interface graphique.
   */
  private void setLabel()
  {
    // Si l'entraneur est dfini.
    if(trainer != null)
    {
      try {
        this.labelErrorNetworkValue.setText(String.format("%.4f", this.trainer.getTestingAverageError()));
      }catch(IllegalStateException e) {
        this.labelErrorNetworkValue.setText("???");
      }
      
      try {
        this.labelTestingClassificationValue.setText(String.format("%.2f", this.trainer.getTestingClassification()) + " %");
      }catch(IllegalStateException e) {
          this.labelTestingClassificationValue.setText("???");
      }
        
      this.labelNbTraining.setText(Integer.toString(this.trainer.getTrainingCount()));
      this.labelNbTrainingVector.setText(Integer.toString(this.trainer.getDataCount()));
      
    }
  }
  
  /**
   * Mthode pour dfinir les images.
   */
  private void setImage()
  {
    // Si le reader est dfini.
    if(this.reader != null)
    {
      this.panelImage.setBufferedImage(this.reader.getDataImage());
      this.panelTrainingImage.setBufferedImage(this.reader.getTrainingDataImage());
      
      // Intgration visuelle des quations des neurones de 1re couche.
      BufferedImage image = this.reader.getTestingDataImage();
      
      // Effectuer les dessins de plan reprsentant les 1re neurones du rseau.
      if(this.trainer != null)
      {
        SNetwork network = this.trainer.getNetwork();
        
        if(network.getNbLayers() != 1)
          SLog.logWriteLine("Message SJFrameColorClassifier --- Prsentement, l'affichage des lignes reprsentant les neurones de la 1re couche ne sont pas affichs, car le rseau possde plusieurs couches.");
        else
        {  
        
          SFullConnectedLayer[] layers = (SFullConnectedLayer[]) network.getLayers();
          
          SFullConnectedLayer first_layer = layers[0];
          
          String[] color = this.reader.getClassificationStringColor();
                  
          // Itrer sur l'ensemble des neurones de la 1re couche.
          for(int i = 0; i < first_layer.getOutputsSize(); i++)
          {
            // quation de la forme : w0*a0 + w1*a1 + b = 0.
            //                         w0*x + w1*y  + b = 0
            double[] eq = first_layer.getEquation(i);
            
            int x1 = 0;
            int y1 = (int) ((-eq[2] / eq[1])*image.getHeight());
            
            int x2 = image.getWidth();
            int y2 = (int) (((-eq[2] - eq[0]*1.0) / eq[1])*image.getHeight());
            
            // Dessiner une ligne pour reprsenter le cot positif.
            double z = ((-eq[2] / eq[1])+1.0)*eq[1] + eq[2];
            if(z > 0)
              SBufferedImageUtil.drawLine(image, x1, y1+3, x2, y2+3, Color.LIGHT_GRAY, 3.0f);
            else
              SBufferedImageUtil.drawLine(image, x1, y1-3, x2, y2-3, Color.LIGHT_GRAY, 3.0f);
            
            // Dessiner la ligne du neurone.
            SBufferedImageUtil.drawLine(image, x1, y1, x2, y2, new Color(Integer.parseInt(color[i])), 3.0f); 
          }
        }
      }      
        this.panelTestingImage.setBufferedImage(image);
        this.panelErrorClassificationImage.setBufferedImage(this.reader.getClassificationErrorTestingImage());
    }
  }
  
  /**
   * Mthode pour effectuer la slection de l'image  classifier.
   */
  private void imageSelection()
  {
    try {
      
      this.reader = new SNeuralImageDataReader(this.textFieldImageFileName.getText());
      
      // Suggestion valide d'un format de rseau admissible.
      this.textFieldNetworkDimension.setText(expressionDefaultNetwork());
      
      // Effacer l'entraneur et le rseau.
      this.trainer = null;
      
      // Effacer l'affichage des caractristiques du rseau.
      this.textPaneNetworkDescription.setText("");
      
      setImage();
      
    }catch(SConstructorException e) {
      SLog.logException(e);
    }
  }
  
  /**
   * Mthode pour faire la slection du rseau et de l'entraineur.
   */
  private void networkSelection()
  {
    try { 
      
      // Cration du rseau  partir d'une expression
      SNetwork network = new SFullConnectedNetwork(this.textFieldNetworkDimension.getText(), DEFAULT_ACTIVATION_FUNCTION_NAME);
      
      // Cration de l'entraneur.
      this.trainer = new STrainer(network, DEFAULT_ERROR_FUNCTION);
            
      // Validation si les donnes d'entranement sont compatible avec l'entraneur.
      this.trainer.addTraining(this.reader.getAllTestingData());
      this.trainer.addTesting(this.reader.getAllTestingData());
      
      // valuer l'erreur initiale sur les donnes.
      this.trainer.computeErrorsTestingData();
      
      // Affichage de la description du rseau.
      this.textPaneNetworkDescription.setText(this.trainer.getNetwork().getDescription());
      
      // Affichage des images.
      this.setLabel();
      this.setImage();
      
    }catch(IllegalArgumentException e) {
      SLog.logException(e);
      this.textPaneNetworkDescription.setText("");
    }catch(NullPointerException e) {
      SLog.logWriteLine("La slection du rseau ne peut pas tre ralise, car l'image n'a pas t slectionne.");
      this.textPaneNetworkDescription.setText("");
    }catch(SConstructorException e) {
      SLog.logException(e);
      this.textPaneNetworkDescription.setText("");
    }
  }
  
  /**
   * Mthode pour obtenir l'expression de la description d'un rseau par dfaut.
   * 
   * @return L'expression dcrivant le rseau.
   */
  private String expressionDefaultNetwork() throws IllegalArgumentException
  {
    if(reader == null)
      throw new IllegalArgumentException("Erreur SJFrameColorClassifier 001 : L'expression du rseau ne peut pas tre dtermin, car le reader est 'null'.");
    
    return this.reader.getDataSize() + " x " + this.reader.getClassificationSize();
  }
}
