/**
 * 
 */
package sim.math;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * La classe <b>SPointFunction</b> reprsente une fonction comme tant une collection de coordonnes (x,y).
 * 
 * @author Simon Vezina
 * @since 2020-01-28
 * @version 2020-02-04
 */
public class SPointFunction {

  /**
   * La variable <b>point_list</b> reprsente la liste de point dfinissant la fonction.
   */
  private final List<SPoint2d> point_list;
  
  /**
   * Constructeur d'une fonction de points.
   */
  public SPointFunction()
  {
    this.point_list = new ArrayList<SPoint2d>();
  }
  
  /**
   * Mthode pour ajouter un point  la fonction.
   * 
   * @param p Le point.
   */
  public void add(SPoint2d p)
  {
    this.point_list.add(p);
  }
  
  /**
   * Mthode pour ajouter un point de coordonne (x,y)  la fonction.
   * 
   * @param x La coordonne x du point.
   * @param y La coordonne y du point.
   */
  public void add(double x, double y)
  {
    add(new SPoint2d(x, y));
  }
  
  /**
   * Mthode pour obtenir la liste de point dcrivant la fonction
   * Cette liste de point sera tri.
   * 
   * @return La liste de point de la fonction.
   */
  public List<SPoint2d> getPoints()
  {
    Collections.sort(this.point_list);
    
    return this.point_list;
  }
  
  /**
   * Mthode pour gnrer une fonction discrte  partir de la liste de points de la fonction de point.
   * Les valeurs discrtes de la fonction discrte qui ne sont pas dfinies dans la liste de point seront
   * calcules  partir d'une interpolation linaire entre deux points situe de chaque ct de l'lment discrte en valuation. 
   * 
   * @param nb Le nombre d'lment discrte qui sera utilis pour dfinir la fonction discrte.
   * @return La fonction discrte.
   */
  public SRealFunction createRealFunction(int nb)
  {
    if(this.point_list.size() < 2)
      throw new IllegalArgumentException("Erreur SPointFunction 001 : Pour crer une fonction rel, il faut avoir un minimum de 2 points dans la fonction de points.");
    
    if(nb < 1)
      throw new IllegalArgumentException("Erreur SPointFunction 002 : Pour crer une fonction rel, il faut un minimum de 2 lments discrets.");
    
    double[] fx = new double[nb];
    
    Collections.sort(this.point_list);
    
    SPoint2d first_point = this.point_list.get(0);
    SPoint2d last_point = this.point_list.get(this.point_list.size()-1);
    
    // L'indice de recherche dans la liste de points.
    int index = 0;
    double dx = SDiscreteFunction.dx(fx, first_point.getX(), last_point.getX());      
    
    // Remplir le tableau de la fonction discrte.
    for(int i = 0; i < fx.length; i++)
    {
      double x = first_point.getX() + (double)i*dx;
      
      // Identifier entre quel points l'index est situ.
      // Cependant, si nous sommes au dernier lment, ne pas permettre de se dplacer dans la liste.
      if(index < this.point_list.size()-2)
        while(x > this.point_list.get(index+1).getX())
          index++;
      
      // Cas particulier  grer :
      // Vrifier si l'indice se retrouve sur le dernier lment du tableau.
      // Si tel est le cas, reculer d'un index, car on peut utiliser l'interpolation pour obtenir la valeur.
      if(index == this.point_list.size()-1)
        index--;
      
      // Facteur d'interpolation.
      double xi = this.point_list.get(index).getX();
      double xf = this.point_list.get(index+1).getX();
        
      double t = (x - xi) / (xf - xi);
      fx[i] = SMath.linearInterpolation(this.point_list.get(index).getY(), this.point_list.get(index+1).getY(), t);
    }
    
    return new SRealFunction(fx, first_point.getX(), last_point.getX());
  }
  
  
  @Override
  public String toString() {
    return "SPointFunction [points=" + point_list + "]";
  }
  
}
