/**
 * 
 */
package sim.math;

import java.util.List;

import sim.exception.SNoImplementationException;
import sim.exception.SRuntimeException;
import sim.util.SWriteable;

/**
 * <p>
 * L'interface <b>SVector</b> reprsente un vecteur mathmatique o des oprations mathmatiques de base peuvent tre effectues.
 * </p>
 * 
 * <p>
 * Les oprations mathmatiques supportes sont les suivantes :
 * <ul>- l'addition de vecteurs.</ul>
 * <ul>- la multiplication par un scalaire d'un vecteur.</ul>
 * </p>
 * 
 * @author Simon Vzina
 * @since 2015-09-22
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public interface SVector extends SWriteable {

  /**
   * Mthode permettant d'effectuer l'addition mathmatique entre deux vecteurs.
   * 
   * @param v Le vecteur  additionner
   * @return Le vecteur rsultat de l'addition des deux vecteurs.
   */
  public SVector add(SVector v);
  
  /**
   * Mthode permettant d'effectuer la multiplication par un scalaire d'un vecteur avec un scalaire.
   * 
   * @param cst La constante scalaire  multiplier avec le vecteur.
   * @return Le vecteur rsultant de la multiplication par un scalaire du vecteur.
   */
  public SVector multiply(double cst);
  
  /**
   * Mthode permettant d'effectuer le <b>produit scalaire</b> entre deux vecteurs.
   * 
   * @param v Le vecteur  mettre en produit scalaire avec le vecteur courant.
   * @return Le rsultat du produit scalaire.
   */
  public double dot(SVector v);
    
  /**
   * <p>
   * Mthode effectuant le calcul de l'interpolation linaire entre deux vecteurs v0 et v1 par le facteur t.
   * L'quation mathmatique correspondant  l'interpolation est
   * <ul> v = v0*(1 - t) + v1*t</ul>
   * o v0 est le vecteur de rfrence et v1 est le vecteur pondr par le facteur t.
   * </p>
   * 
   * @param v0 Le 1ier vecteur de rfrence pondr par 1 - t.
   * @param v1 Le 2ime vecteur pondr par le facteur t.
   * @param t Le paramtre de pondration.
   * @return Le vecteur interpol.
   */
  public static SVector linearInterpolation(SVector v0, SVector v1, double t) 
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * <p>
   * Mthode effectant le calcul de l'interpolation linaire en coordonne barycentrique entre trois vecteurs v0, v1 et v2 par le facteur t1 et t2.
   * L'quation mathmatique correspondant  l'interpolation est
   * <ul> v = v0*(1 - t1 - t2) + v1*t1 + v2*t2</ul>  
   * o v0 est le vecteur de rfrence et v1 est le vecteur pondr par le facteur t1 
   * et v2 est le vecteur pondr par le facteur t2.
   * </p>
   * 
   * @param v0 Le 1ier vecteur de rfrence pondr par 1 - t1 - t2.
   * @param v1 Le 2ime vecteur pondr par t1.
   * @param v2 Le 3ime vecteur pondr par t2.
   * @param t1 Le 1ier paramtre de pondration.
   * @param t2 Le 2ime paramtre de pondration.
   * @return Le vecteur interpol.
   */
  public static SVector linearBarycentricInterpolation(SVector v0, SVector v1, SVector v2, double t1, double t2)
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * <p>
   * Mthode effectant le calcul de l'interpolation linaire en coordonne barycentrique entre plusieurs vecteurs v0, v1, ... et v_n par les facteur t1, t2, ..., t_n.
   * L'quation mathmatique correspondant  l'interpolation est
   * <ul> v = v0*(1 - t1 - t2 - ... - t_n) + v1*t1 + v2*t2 + ... + v_n*t_n</ul>  
   * o v0 est le vecteur de rfrence et les vecteur v_i sont les vecteurs pondrs par les facteurs t_i.
   * </p>
   * 
   * @param vector_list La liste des vecteurs dans l'interpolation. Cette liste contient n+1 vecteur (v0 et les v_i tant de nombre n).
   * @param t_list La liste des facteurs de pondrations des vecteurs. Cette liste contient n facteurs (les n facteurs des n vecteur v_i).
   * @return Le vecteur interpol.
   * @throws SRuntimeException Si le nombre d'lment des listes n'est pas adquat (vector_list.size() != t_list.size()+1). 
   */
  public static SVector linearBarycentricInterpolation(List<SVector> vector_list, List<Double> t_list) throws SRuntimeException
  {
    // Vrifier que les deux listes ont la mme taille
    if(vector_list.size() != t_list.size()+1)
      throw new SRuntimeException("Erreur SVectorUtil 007 : L'interpolation est impossible puisque les deux listes passes en paramtre n'ont les bonnes tailles.");
    
    // Vrification que la liste des vecteurs n'est pas vide
    if(vector_list.isEmpty())
      throw new SRuntimeException("Erreur SVectorUtil 008 : L'interpolation est impossible puisque la liste des vecteurs est vide.");
    
    throw new SNoImplementationException();
    
  }
  
}
