/**
 * 
 */
package sim.nn.function;

/**
 * <p>
 * L'interface <b>SComputeErrorFunctionGradientFiltersWeights</b> permet  une classe d'tre apte  calculer le gradient de la fonction d'erreur par rapport aux poids de convolution d'une couche de neurones.
 * Ce paramtre est associ  la fonction de convolution (<i>convolution function</i>).
 * </p>
 * <p>
 * Ainsi, la classe pourra raliser le calcul 
 * <ul> dC/dW_fdij (en <b>notation tensorielle</b>) </ul>
 * o C est la fonction d'erreur, W est le poids, f est l'indice de profondeur de sortie, d est l'indice de profondeur d'entre,
 * i est l'indice de ligne de la matrice des poids et j est l'indice de colonne de la matrice des poids.
 * 
 * @author Simon Vezina
 * @since 2021-11-26
 * @version 2021-11-26
 */
public interface SComputeErrorFunctionGradientFiltersWeights {

  /**
   * <p>
   * Mthode valuant la drive partielle dC/dW_fdij (en notation tensorielle) de la fonction d'erreur C par rapport au poids W_fdij pour une couche de neurones donnes.
   * </p>
   * <p>
   * Le calcul est effectu  partir de l'erreur propage  la fonction o une activation et une propagation de l'erreur devront tre ralises pralablement.
   * </p>
   * 
   * @param f L'indice de profondeur de sortie de la couche (correspondant un numro du filtre).
   * @param d L'indice de profondeur d'entre de la couche.
   * @param i L'indice de la ligne du filtre.
   * @param j L'indice de la colonne du filtre.
   * @return La drive partielle de la fonction d'erreur C par rapport au poids de convolution W_fdij.
   */
  public double dCdW(int f, int d, int i, int j);
  
}
