/**
 * 
 */
package sim.nn.function;

/**
 * <p>
 * L'interface <b>SComputeErrorFunctionGradientWeights</b> permet  une classe d'tre apte  calculer le gradient de la fonction d'erreur par rapport aux poids d'une couche de neurones.
 * Ce paramtre est associ  la fonction d'agrgation (<i>aggregation function</i>).
 * </p>
 * <p>
 * Ainsi, la classe pourra raliser le calcul 
 * <ul> dC/dW_uv (en <b>notation matricielle</b>) ou dC/dW_vu (en <b>notation tensorielle</b>) </ul>
 * o C est la fonction d'erreur, W est le poids, v est l'indice d'entre du poids (indice de colonne de la matrice) et u est l'indice de sortie du poids (indice de la ligne de la matrice).
 * 
 * @author Simon Vezina
 * @since 2021-11-26
 * @version 2021-11-26
 */
public interface SComputeErrorFunctionGradientWeights {

  /**
   * <p>
   * Mthode valuant la drive partielle dC/dW_vu (en notation tensorielle) de la fonction d'erreur C par rapport au poids W_vu pour une couche de neurones donnes.
   * Cette mthode ralise le calcul pour une fonction d'ordre 1 uniquement comme il est requis pour la fonction d'agrgation de la <u>somme pondre</u> (<i>weighted sum</i>).
   * </p>
   * <p>
   * Le calcul est effectu  partir de l'erreur propage  la fonction d'agrgation o une activation et une propagation de l'erreur devront tre ralises pralablement.
   * </p>
   * 
   * @param v L'indice d'entre du poids (indice de la colonne en matrice).
   * @param u L'indice de sortie du poids (indice de la ligne en matrice).
   * @return La drive de la fonction d'erreur C par rapport au poids W_vu.
   */
  public default double dCdW(int v, int u)
  {
    return dCdW(v, u, 1);
  }
    
  /**
   * <p>
   * Mthode valuant la drive de la fonction d'erreur par rapport au poids W_vu (notation tensorielle) d'ordre o pour une couche de neurones donnes.
   * </p>
   * <p>
   * Le calcul est effectu  partir de l'erreur propage  la fonction d'agrgation o une activation et une propagation de l'erreur devront tre ralises pralablement.
   * </p>
   * @param v L'indice d'entre du poids (indice de la colonne en matrice).
   * @param u L'indice de sortie du poids (indice de la ligne en matrice).
   * @param o L'ordre du poids (entre 1 et n).
   * @return La drive de la fonction d'erreur C par rapport au poids W_vu pour l'ordre o.
   * @throws IllegalArgumentException Si l'ordre du poids n'est pas adquat pour la fonction.
   */
  public double dCdW(int v, int u, int o) throws IllegalArgumentException;
    
}
