/**
 * 
 */
package sim.nn.function;

import sim.nn.SActivable;
import sim.util.SArrays;

/**
 * La classe abstraite <b>SMemoryNeuralFunction</b> reprsente une structure de donne permettant 
 * l'allocation de la mmoire pour une fonction neuronale.
 * 
 * @author Simon Vezina
 * @since 2019-02-22
 * @version 2022-01-05
 */
public abstract class SMemoryNeuralFunction implements SActivable, SNeuralFunction {
 
  /**
   * La variable <b>forward_inputs</b> reprsente les valeurs des neurones en entre de la fonction lors de l'application <u>vers l'avant</u> de la fonction.
   */
  protected double[] forward_inputs;
  
  /**
   * La variable <b>forward_outputs</b> reprsente le vecteur de sortie de la fonction dans le sens <u>vers l'avant</u>.  
   */
  final protected double[] forward_outputs;
  
  /**
   * La variable <b>backward_input</b> reprsente les valeurs des neurones en entre de la fonction lors de l'application <u>vers l'arrire</u> de la fonction.
   */
  protected double[] backward_inputs;
  
  /**
   * La variable <b>backward_outputs</b> reprsente le vecteur de sortie de la fonction dans le sens <u>vers l'arrire</u>.  
   */
  final protected double[] backward_outputs;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur d'une fonction de neurones de base ayant le mme nombre de neurones en entre qu'en sortie.
   * 
   * @param neurons Le nombre de neurones.
   */
  public SMemoryNeuralFunction(int neurons)
  {
    this(neurons, neurons);
  }
  
  /**
   * Constructeur d'une fonction de neurones de base 
   * 
   * @param inputs_size Le nombre de neurones en entre de la fonction.
   * @param outputs_size Le nombre de neurones en sortie de la fonction.
   */
  public SMemoryNeuralFunction(int inputs_size, int outputs_size)
  {
    this.forward_inputs = null;
    this.forward_outputs = new double[outputs_size];
    
    this.backward_inputs = null;
    this.backward_outputs = new double[inputs_size];
  }
  
  //------------
  // MTHODES //
  //------------
  
  @Override
  public int getInputsSize()
  {
    return this.backward_outputs.length;
  }

  @Override
  public int getOutputsSize()
  {
    return this.forward_outputs.length;
  }
       
  @Override
  public String toString()
  {
    return getFunctionName();
  }
  
  @Override
  public boolean equals(Object obj) 
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    
    SMemoryNeuralFunction other = (SMemoryNeuralFunction) obj;
    
    if (!SArrays.nearlyEquals(backward_inputs, other.backward_inputs))
      return false;
    if (!SArrays.nearlyEquals(backward_outputs, other.backward_outputs))
      return false;
    if (!SArrays.nearlyEquals(forward_inputs, other.forward_inputs))
      return false;
    if (!SArrays.nearlyEquals(forward_outputs, other.forward_outputs))
      return false;
    
    return true;
  }
  
}
