/**
 * 
 */
package sim.nn.function.activation;

import sim.exception.SConstructorException;
import sim.exception.SNaNValueException;
import sim.nn.function.SNeuralFunction;
import sim.nn.function.SNeuralMath;

/**
 * La classe <b>SSoftmaxFunction</b> reprsente la fonction d'activation <i>softmax</i>.
 * Cette fonction d'activation de type <i>full connected</i> permet d'interprter toutes les activations d'une couche comme tant une distribution de probabilit (dont la somme sera gale  1).
 *  
 * @author Simon Vzina
 * @since 2018-05-08
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public class SSoftmaxFunction extends SFullActivationFunction {

  /**
   * Constructeur de la fonction d'activation softmax.
   * 
   * @param neurons Le nombre de neurones dans la fonction.
   * @throws SConstructorException S'il y a eu une erreur lors de la fonction de l'objet.
   */
  public SSoftmaxFunction(int neurons) throws SConstructorException
  {
    super(neurons);
  }
  
  @Override
  public double forward(int u)
  {
    return SNeuralMath.softmax(this.forward_inputs, u);
  }
  
  @Override
  protected double[] forwardPass() throws SNaNValueException
  {
    return super.forwardPass();
    
  }
  
  @Override
  public double dAdZ(int u, int k)
  {
    return SNeuralMath.derivateSoftmax(this.forward_inputs, u, k);
    
      
  }

  @Override
  public String getFunctionName() 
  {
    return SNeuralFunction.SOFTMAX_FUNCTION;
  }
  
}
