/**
 * 
 */
package sim.nn.function.activation;

import sim.exception.SNoImplementationException;
import sim.nn.function.SNeuralFunction;

/**
 * La classe <b>SSwishFunction</b> reprsente la fonction d'activation portant le nom de <i>swish</i>.
 * 
 * @author Simon Vezina
 * @since 2021-12-05
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public class SSwishFunction extends SActivationFunction {

  /**
   * La variable <b>sigma</b> reprsente les calcul de 1 / 1 + exp(-k*z) pour l'ensemble des neurones z de la fonction.
   * Ces valeurs seront dtermines lors de l'activation de la fonction.
   */
  private final double[] sigma;
  
  /**
   * Constructeur de la fonction d'activation swish contenant 1 type de paramtre.
   * 
   * @param neurons Le nombre de neurones dans la fonction.
   */
  public SSwishFunction(int neurons) 
  {
    super(neurons);
    
    // Espace mmoire pour enregistrer la valeur de sigma afin d'acclrer le processus de calcul.
    this.sigma = new double[neurons];
  }
  
  @Override
  public boolean asParameter()
  {
    return true;
  }
  
  @Override
  public String getFunctionName()
  {
    return SNeuralFunction.SWISH_FUNCTION;
  }

  @Override
  public int getNbParameterType()
  {
    return 1;
  }
  
  @Override
  protected double forward(int u)
  {
    throw new SNoImplementationException();
    
  }

  @Override
  public double dAdZ(int u)
  {
    throw new SNoImplementationException();
    
  }
  
  @Override
  public double dAdK(int i, int u) throws IllegalArgumentException
  {
    // Validation de du type de paramtre, car la fonction swish ne contient qu'un seul paramtre.
    if(i != 0)
      throw new IllegalArgumentException("Erreur SSwishFunction 002 : La fonction swish ne possde qu'un seul type de paramtre d'indice 0 (indice demand tant " + i + ").");
    
    throw new SNoImplementationException();
    
  }
  
  @Override
  public boolean isActivationFunctionWithParameters()
  {
    return true;
  }
  
}
