/**
 * 
 */
package sim.nn.function.activation;

import sim.exception.SConstructorException;
import sim.nn.function.SNeuralFunction;
import sim.nn.function.SNeuralMath;

/**
 * <p>
 * La classe <b>STanhFunction</b> reprsente la fonction d'activation portant le nom de <b>Tanh</b> correspondant  la fonction tangente hyperbolique.
 * </p>
 * <p>
 * Les valeurs de sortie de cette fonction d'activation est dans l'intervalle [-1, 1].
 * </p>
 * 
 * @author Simon Vzina
 * @since 2018-05-25
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public class STanhFunction extends SActivationFunction {

  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur de la fonction d'activation tangente hyperbolique.
   * 
   * @param neurons Le nombre de neurones.
   * @throws SConstructorException S'il y a eu erreur lors de la construction de la fonction.
   */
  public STanhFunction(int neurons) throws SConstructorException
  {
    super(neurons);
  }

  //------------
  // MTHODES //
  //------------
  
  @Override
  public double forward(int u) 
  {
    return SNeuralMath.tanh(this.forward_inputs[u]);
  }

  
  @Override
  public double dAdZ(int u)
  {
    return SNeuralMath.derivateTanh(this.forward_inputs[u]);  
    
  }
  
  @Override
  public String getFunctionName() 
  {
    return SNeuralFunction.TANH_FUNCTION;
  }
  
}
