/**
 * 
 */
package sim.nn.function.aggregation;

import sim.exception.SNoImplementationException;
import sim.nn.function.SNeuralFunction;
import sim.nn.function.SNeuralMath;

/**
 * La classe <b>SWeightedSumFunction</b> reprsente le fonction d'agrgation de la somme pondre.
 * 
 * @author Simon Vezina
 * @since 2018-08-05
 * @version 2022-09-14 (Version labo : Apprentissage par rseau de neurones)
 */
public class SWeightedSumFunction extends SAggregationFunction {

  /**
   * La constante <b>ORDER</b> reprsente l'ordre de la fonction d'agrgation de la somme pondre gal  {@value}.
   */
  public static final int ORDER = 1;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur de la fonction d'agrgation de la somme pondre.
   * 
   * @param input Le nombre d'entre dans la fonction.
   * @param output Le nombre de sortie dans la fonction.
   */
  public SWeightedSumFunction(int input, int output)
  {
    super(input, output);
  }

  //------------
  // MTHODES //
  //------------
       
   /**
    * <p>
    * Mthode pour obtenir la drive de l'agrgation Z_u par rapport au poids W_uv.
    * </p>
    * <p>
    * Il est important de prciser l'interprtation de deux notations :
    * <ul>- Matrice : d(Z_u) / d(W_uv) o u est l'indice de ligne et v est l'indice de colonne.</ul>
    * <ul>- Tenseur : d(Z_u) / d(W_vu) o v est l'indice d'entre et u est l'indice de sortie.</ul>
    * </p> 
    * 
    * @param u L'indice de sortie (tenseur) ou de ligne (matrice). C'est l'indice Z_u.
    * @param v L'indice d'entre (tenseur) ou colonne (matrice). C'est le 2e indice de W_uv.
    * @return La drive de Z par rapport  W.
    */
   public double dZdW(int u, int v)
   {
     throw new SNoImplementationException();
   }
     
   @Override
   public double dZdW(int u, int v, int o) throws IllegalArgumentException
   {
     if(o == 1)
       return this.dZdW(u, v);
     else
       throw new IllegalArgumentException("Erreur SWeightedSumFunction 002 : L'ordre o = " + o + " n'est pas admissible pour cette fonction.");
   }
   
   @Override
   public double dZdA(int u, int v)
   {
     throw new SNoImplementationException();
     
   }
      
   @Override
   public double dZdB(int u)
   {
     throw new SNoImplementationException();
     
   }
   
   @Override
   public double dCdW(int v, int u)
   {
     throw new SNoImplementationException();
     
   }
   
   @Override
   public double dCdW(int v, int u, int o) throws IllegalArgumentException
   {
     if(o != 1)
       throw new IllegalArgumentException("Erreur SWeightedSumFunction 001 : La fonction d'agrgation de type somme pondre ne possde que des poids d'ordre 1. Ainsi, l'ordre o = " + o + " n'est pas support.");
     
     return dCdW(v, u);
   }
   
   @Override
   protected double forward(int u)
   {
     return SNeuralMath.weightedSum(u, this.forward_inputs, this.W.getMatrix(), this.B.getVector());
   }
      
   @Override
   public int getOrder() 
   {
     return SWeightedSumFunction.ORDER;
   }
   
   @Override
   public String getFunctionName() 
   {
     return SNeuralFunction.WEIGHTED_SUM_FUNCTION;
   }
  
}
