/**
 * 
 */
package sim.nn.function.error;

import sim.nn.function.SNeuralFunction;
import sim.nn.function.SNeuralMath;

/**
 * La classe <b>SEnergyErrorFunction</b> reprsente la fonction de cot de l'erreur quadratique (surnomme <i>nergie</i>).
 * 
 * @author Simon Vzina
 * @since 2018-05-05
 * @version 2022-01-08
 */
public class SEnergyErrorFunction extends SErrorFunction {

  /**
   * Constructeur de la fonction d'erreur quadratique portant le nom d'nergie.
   * 
   * @param neurons Le nombre de neurones dans la fonction d'erreur.
   */
  public SEnergyErrorFunction(int neurons)
  {
    super(neurons);
  }
  
  @Override
  public double dCdA(int u)
  {
    return SNeuralMath.derivateQuadraticError(this.forward_inputs[u], this.y[u]);    
  }

  @Override
  public double forward(int u) 
  {
    return SNeuralMath.quadraticError(this.forward_inputs[u], this.y[u]);
  }

  @Override
  public String getFunctionName()
  {
    return SNeuralFunction.ENERGY_ERROR_FUNCTION;
  }
  
}
