/**
 * 
 */
package sim.nn.function.error;

import sim.util.SStringUtil;

/**
 * La classe <b>SErrorFunctionBuilder</b> permet la construction de diffrentes fonction d'erreur utilises dans l'apprentissage d'un rseau de neurones.
 * 
 * @author Simon Vzina
 * @since 2018-05-05
 * @version 2021-09-11
 */
public class SErrorFunctionBuilder {

  /**
   * <p>
   * La constante <b>FUNCTIONS</b> reprsente la liste des noms des fonctions d'erreurs qui sont supportes.
   * </p>
   * <p>
   * On y retrouve la liste suivante :
   * <ul>Energy</ul>
   * <ul>Linear</ul>
   * <ul>Entropy</ul>
   * </p>
   */
  public final static String[] ERROR_FUNCTIONS = { "linear", "energy", "cross-entropy"};
  
  public final static int LINEAR_CODE = 0;
  public final static int ENERGY_CODE = 1;
  public final static int CROSS_ENTROPY_CODE = 2;
  
  /**
   * Mthode effectuant la construction de la fonction d'erreur.
   * 
   * @param function_name Le nom de la fonction d'erreur.
   * @param nb_neurons Le nombre de neurones.
   * @return La fonction d'erreur.
   * @throws IllegalArgumentException Si un paramtre n'est pas adquat.
   */
  public static SErrorFunction build(String function_name, int nb_neurons) throws IllegalArgumentException
  {
    // Dtermination de la fonction d'erreur.
    if(function_name.equalsIgnoreCase(ERROR_FUNCTIONS[ENERGY_CODE]))
      return new SEnergyErrorFunction(nb_neurons);
    
    if(function_name.equalsIgnoreCase(ERROR_FUNCTIONS[LINEAR_CODE]))
      return new SLinearErrorFunction(nb_neurons);
    
    if(function_name.equalsIgnoreCase(ERROR_FUNCTIONS[CROSS_ENTROPY_CODE]))
      return new SCrossEntropyErrorFunction(nb_neurons);
    
    // Erreur, car il n'y a pas eu reconnaissance du nom de la fonction.
    throw new IllegalArgumentException("Erreur SErrorFunctionBuilder 001 : La fonction '" + function_name + "' n'est pas supporte. Voici la liste disponible : " + SStringUtil.join(ERROR_FUNCTIONS, ", "));
  }
  
}
