/**
 * 
 */
package sim.nn.function.filtering;

import sim.nn.function.SNeuralFunction;
import sim.util.SStringUtil;

/**
 * La classe <b>SPoolingFunctionBuilder</b> reprsente un constructeur de fonction de pooling.
 * 
 * @author Simon Vezina
 * @since 2021-10-18
 * @version 2022-01-08
 */
public class SPoolingFunctionBuilder {

  public final static String[] POOLING_FUNCTIONS = { 
    SNeuralFunction.MAX_POOLING_FUNCTION, SNeuralFunction.ABSOLUTE_MAX_POOLING_FUNCTION
  };
  
  public final static int MAX_POOLING_CODE = 0;
  public final static int ABSOLUTE_MAX_POOLING_FUNCTION_CODE = 1;
  
  
  /**
   * ...
   * 
   * @param function_name
   * @param nb_neurons
   * @return
   * @throws IllegalArgumentException
   */
  public static SPoolingFunction build(String function_name, int inputs, int depth, int filter_offset) throws IllegalArgumentException
  {
    // Dtermination de la fonction de pooling.
    if(function_name.equalsIgnoreCase(POOLING_FUNCTIONS[MAX_POOLING_CODE]))
      return new SMaxPoolingFunction(inputs, depth, filter_offset);
    
    // Dtermination de la fonction de pooling.
    if(function_name.equalsIgnoreCase(POOLING_FUNCTIONS[ABSOLUTE_MAX_POOLING_FUNCTION_CODE]))
      return new SAbsoluteMaxPoolingFunction(inputs, depth, filter_offset);
    
    // Erreur, car il n'y a pas eu reconnaissance du nom de la fonction.
    throw new IllegalArgumentException("Erreur SPoolingFunctionBuilder 001 : La fonction '" + function_name + "' n'est pas supporte. Voici la liste disponible : " + SStringUtil.join(POOLING_FUNCTIONS, ", "));
  }
  
}
