/**
 * 
 */
package sim.nn.function.regularization;

import sim.exception.SNoImplementationException;
import sim.nn.function.SAbstractNeuralFunction;
import sim.nn.function.SNeuralFunction;

/**
 * <p>
 * La classe <b>SDropoutFunction</b> reprsente la fonction de rgularisation portant le nom de <i>drop out</i>.
 * </p>
 * <p>
 * Cette fonction a pour but d'invalider alatoirement la sortie de certains neurones favorisant statistiquement
 * un meilleure rpartition des effets de tous les neurones lors de l'apprentissage. Ainsi, un neurone
 * ne pourra pas se faire sur-entraner au dtriment des autres. 
 * </p>
 * 
 * @author Simon Vezina
 * @since 2019-10-24
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones) 
 */
public class SDropoutFunction extends SAbstractNeuralFunction {

  /**
   * La constante <b>DEFAULT_DROPPING_PROBABILITY</b> reprsente la probabilit du dropout par dfaut tablie  {@value}.
   */
  public static final double DEFAULT_DROPPING_PROBABILITY = 0.25;
  
  /**
   * La constante <b>DEFAULT_DROPPING</b> reprsente l'tat par dfaut de la fonction correspondant  l'application <b>{@value}</b> du 'dropping'.
   */
  public static final boolean DEFAULT_DROPPING = false;

  /**
   * Le variable globale <b>is_dropping</b> reprsente l'tat de la fonction  l'effet si la fonction va raliser ou non le 'dropping'.
   */
  private static boolean is_dropping = DEFAULT_DROPPING;
  
  /**
   * La variable <b>dropping_probability</b> reprsente la probabilit de faire le drop.
   */
  protected double dropping_probability;
  
  /**
   * Constructeur d'une fonction de rgularisation <i>dropout</i>.
   * 
   * @param neuronsLe nombre de neurones.
   */
  public SDropoutFunction(int neurons)
  {
    super(neurons);
    
    this.dropping_probability = DEFAULT_DROPPING_PROBABILITY;
  }

  /**
   * Mthode ralisant l'activation de la mthode <i>dropout</i> propageant une donne nulle sous une certaine probabilit.
   * 
   * @param A L'entre de la fonction <i>dropout</i>.
   * @param p La probabilit du <i>drop</i> (rsultat gnr tant 0).
   * @return L'activation de la la mthode <i>dropout</i>.
   */
  public double[] drop(double[] A, double p) throws IllegalArgumentException
  {
    // Validation de la probabilit.
    if(p < 0.0 || p > 1.0)
      throw new IllegalArgumentException("Erreur SDropoutFunction 001 : La probabilit p = " + p + " n'est pas contenu entre 0 et 1.");
    
    // Conserver la rfrence du paramtre.
    this.dropping_probability = p;
    
    // Effectuer l'activation de la fonction.
    return this.forwardPropagation(A);  
  }
    
  @Override
  protected double forward(int u)
  {
    throw new SNoImplementationException();
          
  }

  @Override
  protected double backward(int u) 
  {
    throw new SNoImplementationException();
    
  }

  @Override
  public String getFunctionName()
  {
    return SNeuralFunction.DROPOUT_FUNCTION;
  }

  /**
   * Mthode pour dfinir l'tat du dropping des fonction de dropout.
   * Lors de l'entranement, il faut le mettre  <b>true</b>.
   * Lors de la validation, il faut le mettre  <b>false</b>.
   * 
   * @param dropping L'tat du dropping.
   */
  public static void setDropping(boolean dropping)
  {
    is_dropping = dropping;
  }
  
}
