/**
 * 
 */
package sim.nn.function.regularization;

/**
 * La classe <b>SRunningMeanVariance</b> reprsente la moyenne et la variance en volution d'une couche de neurones.
 * 
 * @author Simon Vezina
 * @since 2019-04-16
 * @version 2022-01-08
 */
public class SRunningMeanVariance {

  /**
   * La constante <b>RUNNING_OLD_VALUE_RATIO</b> reprsente le ratio associ  l'ancienne valeur qui est gal  {@value}.
   */
  public static final double RUNNING_OLD_VALUE_RATIO = 0.9;
  
  /**
   * La constante <b>RUNNING_NEW_VALUE_RATIO</b> reprsente le ratio associ  la nouvelle valeur qui est gal  {@value}.
   */
  public static final double RUNNING_NEW_VALUE_RATIO = 1.0 - RUNNING_OLD_VALUE_RATIO;
  
  /**
   * ...
   */
  private static final double DEFAULT_MEAN = 0.0;
  
  /**
   * ...
   */
  private static final double DEFAULT_VARIANCE = 1.0;
  
  //-------------
  // VARIABLES //
  //-------------
  
  private final double[] mean;
  private final double[] variance;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * ...
   * 
   * @param neurons
   */
  public SRunningMeanVariance(int neurons)
  {
    this.mean = new double[neurons];
    this.variance = new double[neurons];
    
    for(int i = 0; i < neurons; i++)
    {
      this.mean[i] = DEFAULT_MEAN;
      this.variance[i] = DEFAULT_VARIANCE;
    }
    
  }
  
  /**
   * Mthode pour obtenir le nombre de neurones associ aux calculs des moyennes et variances.
   * 
   * @return Le nombre de neurones.
   */
  public int getNeurons()
  {
    return this.mean.length;
  }
  
  /**
   * ...
   * 
   * @param u
   * @return
   */
  public double getMean(int u)
  {
    return this.mean[u];
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getVariance(int u)
  {
    return this.variance[u];
  }
  
  /**
   * ...
   * 
   * @param u
   * @param mean
   */
  public void updateMean(int u, double mean)
  {
    this.mean[u] =  RUNNING_OLD_VALUE_RATIO*this.mean[u] + RUNNING_NEW_VALUE_RATIO*mean; 
  }
  
  /**
   * ...
   * 
   * @param u
   * @param variance
   */
  public void updateVariance(int u, double variance)
  {
    this.variance[u] =  RUNNING_OLD_VALUE_RATIO*this.variance[u] + RUNNING_NEW_VALUE_RATIO*variance; 
  }
  
}
