/**
 * 
 */
package sim.nn.gradient;

import sim.nn.function.SComputeErrorFunctionGradientScale;
import sim.nn.function.SComputeErrorFunctionGradientShift;
import sim.nn.layer.SLayer;
import sim.nn.parameter.SContainScale;
import sim.nn.parameter.SContainShift;
import sim.nn.parameter.SScale;
import sim.nn.parameter.SShift;

/**
 * La classe <b>SBatchConvolutionGradientLayer</b> reprsente une couche contenant les gradients de la fonction d'erreur pour une couche de type <i>batch</i> de convolution. 
 * 
 * @author Simon Vezina
 * @since 2021-11-10
 * @version 2021-12-31
 */
public class SBatchConvolutionGradientLayer extends SConvolutionGradientLayer implements SContainScale, SContainShift
{

  /**
   * ...
   */
  protected final SScale grad_SC;
  
  /**
   * ...
   */
  protected final SShift grad_SH;
  
  /**
   * ...
   * 
   * @param outputs_size Le nombre de neurones en sortie de la couche.
   * @param filter_offset La taille du filtre en ligne et colonne.
   * @param input_depth Le nombre de matrice par filtre (la profondeur d'entre).
   * @param output_depth Le nombre de filtre dans la convolution (la profondeur de sortie).
   */
  public SBatchConvolutionGradientLayer(int outputs_size, int filter_offset, int input_depth, int output_depth, String activation_function_name)
  {
    super(outputs_size, filter_offset, input_depth, output_depth, activation_function_name);
   
    this.grad_SC = new SScale(outputs_size, false);
    this.grad_SC.clear();
    
    this.grad_SH = new SShift(outputs_size, false);
    this.grad_SH.clear();
  }

  //------------
  // MTHODES //
  //------------
  
  @Override
  public SScale getScale() 
  {
    return this.grad_SC;
  }
  
  @Override
  public SShift getShift()
  {
    return this.grad_SH;
  }
    
  @Override
  public void add(SLayer layer)
  {
    // valuer le gradient pour les paramtres hrits.
    super.add(layer);
    
    SComputeErrorFunctionGradientScale dCdSC = (SComputeErrorFunctionGradientScale) layer;
    SComputeErrorFunctionGradientShift dCdSH = (SComputeErrorFunctionGradientShift) layer;
    
    double value;
    
    // Itrer sur l'ensemble des neurones de sortie de la couche.
    for(int u = 0; u < this.grad_SC.getNeurons(); u++)
    {
      // Dfinir le gradient du scale pour le neurone u.
      value = this.grad_SC.get(u) + dCdSC.dCdSC(u);
      this.grad_SC.set(u, value);
      
      // Dfinir le gradient du shift pour le neurone u.
      value = this.grad_SH.get(u) + dCdSH.dCdSH(u);
      this.grad_SH.set(u, value);
    }
    
  }
  
  @Override
  public void add(SGradientLayer gradient)
  {
    super.add(gradient);
    
    this.grad_SC.add(((SContainScale)gradient).getScale());
    this.grad_SH.add(((SContainShift)gradient).getShift());    
  }
    
  @Override
  public void clear()
  {
    super.clear();
    
    this.grad_SC.clear();
    this.grad_SH.clear();
  }
  
  @Override
  public void multiply(double v) 
  {
    super.multiply(v);
    
    this.grad_SC.multiply(v);
    this.grad_SH.multiply(v);
  }

  @Override
  public void multiply(SGradientLayer gradient) 
  {
    super.multiply(gradient);
    
    this.grad_SC.multiply(((SContainScale)gradient).getScale());
    this.grad_SH.multiply(((SContainShift)gradient).getShift());
  }

  @Override
  public boolean equals(Object obj) {
    if (this == obj)
      return true;
    
    if (!super.equals(obj))
      return false;
    
    if (getClass() != obj.getClass())
      return false;
    
    SBatchConvolutionGradientLayer other = (SBatchConvolutionGradientLayer) obj;
    if (grad_SC == null) {
      if (other.grad_SC != null)
        return false;
    } else if (!grad_SC.equals(other.grad_SC))
      return false;
    if (grad_SH == null) {
      if (other.grad_SH != null)
        return false;
    } else if (!grad_SH.equals(other.grad_SH))
      return false;
    
    return true;
  }
 
}
