/**
 * 
 */
package sim.nn.gradient;

import sim.nn.layer.SLayer;

/**
 * L'interface <b>SGradientLayer</b> reprsente une couche de gradient de fonction d'erreur.
 * 
 * @author Simon Vezina
 * @since 2019-08-17
 * @version 2020-01-12
 */
public interface SGradientLayer {

  /**
   * Mthode pour ajouter au gradient la contribution d'un autre gradient.
   * 
   * @param gradient La couche  ajouter appartenant  un gradient.
   * @throws IllegalArgumentException Si la couche ne peut pas tre applique dans le calcul.
   */
  public void add(SGradientLayer gradient) throws IllegalArgumentException;
  
  /**
   * Mthde pour ajouter au gradient la contribution d'une couche de neurones par l'usage de la drive de la fonction d'erreur par rapport aux diffrents paramtres de la couches de neurones.
   * Pour que ce calcul soit effectu avec succs, il faut que la couche ait t pralablement active et que la propagation de l'erreur ait t ralise.
   * 
   * @param layer La couche de neurones.
   * @throws IllegalArgumentException Si la couche ne peut pas tre applique dans le calcul.
   */
  public void add(SLayer layer) throws IllegalArgumentException;
  
  /**
   * Mthode pour mettre  zro l'ensemble des valeurs des paramtres.
   */
  public void clear();

  /**
   * Mthode pour multiplier l'ensemble des paramtres du gradient par une constante.
   * 
   * @param v La constante.
   */
  public void multiply(double v);

  /**
   * Mthode pour multiplier l'ensemble des paramtres du gradient avec les composantes d'un autre gradient.
   * La multiplication sera effectue composante par composante.
   * 
   * @param gradient Le gradient  multiplier.
   */
  public void multiply(SGradientLayer gradient);
  
}
