package sim.nn.layer;

import java.util.List;

import sim.nn.function.regularization.SBatchDropoutFunction;
import sim.util.SStringUtil;

/**
 * La classe <b>SBatchDropoutLayer</b> reprsente une couche de neurones contenant la fonction dropout pouvant tre active par <i>batch</i>.
 * 
 * @author Simon Vezina
 * @since 2021-11-19
 * @version 2022-01-08
 */
public class SBatchDropoutLayer implements SBatchLayer {

  /**
   * ...
   */
  protected final SBatchDropoutFunction D;
  
  /**
   * ...
   */
  protected double dropping_probability;
  
  /**
   * ...
   * 
   * @param neurons
   * @param dropping_probability
   */
  public SBatchDropoutLayer(int neurons, double dropping_probability)
  {
    this.D = new SBatchDropoutFunction(neurons);
    
    this.dropping_probability = dropping_probability;
  }

  /**
   * ...
   * 
   * @return
   */
  public double getDropout()
  {
    return this.dropping_probability;
  }
  
  @Override
  public int getInputsSize() 
  {
    return this.D.getInputsSize();
  }

  @Override
  public int getOutputsSize() 
  {
    return this.D.getOutputsSize();
  }

  @Override
  public String getLayerName()
  {
    return SLayer.BATCH_DROPOUT_LAYER;
  }
  
  @Override
  public String getDescription()
  {
    return this.getLayerName() + " [ Dimension = " + this.getDimension() + ", D = " + this.D + " (" + this.dropping_probability +")]" + SStringUtil.END_LINE_CARACTER;
  }

  @Override
  public SLayer copyWithSameParameter()
  {
    return new SBatchDropoutLayer(this.getInputsSize(), this.dropping_probability);
  }

  @Override
  public List<double[]> batchForwardPropagation(int b, List<double[]> batch_inputs) throws IllegalArgumentException 
  {
    return this.D.batchDrop(b, batch_inputs, this.dropping_probability);
  }

  @Override
  public List<double[]> batchBackPropagation(int b, List<double[]> batch_delta) throws IllegalArgumentException 
  {
    return this.D.batchBackPropagation(b, batch_delta);
  }

  @Override
  public int getBatchSize()
  {
    return this.D.getBatchSize();
  }

  @Override
  public void setBatchSize(int b) throws IllegalArgumentException 
  {
    this.D.setBatchSize(b);
  }

  @Override
  public int getMaximumBatchSize() 
  {
    return this.D.getMaximumBatchSize();
  }

  @Override
  public boolean upgradeMaximumBatchSize(int size)
  {
    return this.D.upgradeMaximumBatchSize(size);
  }

  @Override
  public String toString()
  {
    return getDescription();
  }
  
}
