/**
 * 
 */
package sim.nn.layer;

import java.util.List;

import sim.nn.function.filtering.SBatchPoolingFunction;
import sim.util.SStringUtil;

/**
 * La classe <b>SBatchPoolingLayer</b> reprsente une couche de neurones contenant une fonction de pooling pouvant tre active par <i>batch</i>.
 * 
 * @author Simon Vezina
 * @since 2021-11-19
 * @version 2022-01-08
 */
public class SBatchPoolingLayer implements SBatchLayer {
  
  /**
   * ...
   */
  protected final SBatchPoolingFunction P;
  
  /**
   * ...
   * 
   * @param inputSize
   * @param inputDepth
   * @param filterOffset
   * @param poolingFunctionName
   */
  public SBatchPoolingLayer(int inputSize, int inputDepth, int filterOffset, String poolingFunctionName) 
  {
    // Nous allons mettre un Stride gale  la taille du filtre.
    this.P = new SBatchPoolingFunction(inputSize, inputDepth, filterOffset, filterOffset, poolingFunctionName);
  }

  @Override
  public List<double[]> batchForwardPropagation(int b, List<double[]> batch_inputs) throws IllegalArgumentException 
  {
    return this.P.batchPooling(b, batch_inputs);
  }

  @Override
  public List<double[]> batchBackPropagation(int b, List<double[]> batch_delta) throws IllegalArgumentException 
  {
    return this.P.batchBackPropagation(b, batch_delta);
  }

  @Override
  public int getBatchSize() 
  {
    return this.getBatchSize();
  }
  
  @Override
  public int getInputsSize() 
  {
    return this.P.getInputsSize();
  }

  @Override
  public int getOutputsSize() 
  {
    return this.P.getOutputsSize();
  }

  @Override
  public int getInputsDepth() 
  {
    return this.P.getInputsDepth();
  }
  
  @Override
  public int getOutputsDepth() 
  {
    return this.P.getOutputsDepth();
  }
  
  /**
   * Mthode pour obtenir la taille du filtre carr de la fonction de regroupement.
   * 
   * @return La taille du filtre.
   */
  public int getFilterOffset()
  {
    return this.P.getFilterOffset();
  }
  
  /**
   * Mthode pour obtenir le nom de la fonction de regroupement de la couche.
   * 
   * @return Le nom de la fonction de regroupement.
   */
  public String getPoolingFunctionName()
  {
    return this.P.getFunctionName();
  }
  
  @Override
  public String getDimension() 
  {
    return this.P.getDimension();
  }
  
  @Override
  public String getLayerName()
  {
    return SLayer.BATCH_POOLING_LAYER;
  }
  
  @Override
  public String getDescription() 
  {
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(this.getLayerName() + " [ Dimension = " + this.getDimension() + ", P = " + P + "]" + SStringUtil.END_LINE_CARACTER);
    
    return buffer.toString();
  }

  @Override
  public SLayer copyWithSameParameter() 
  {
    return new SBatchPoolingLayer(this.getInputsSize(), this.getInputsDepth(), this.P.getFilterOffset(), this.P.getFunctionName());
  }

  @Override
  public String toString()
  {
    return getDescription();
  }

  @Override
  public void setBatchSize(int b) throws IllegalArgumentException 
  {
    this.P.setBatchSize(b);
  }

  @Override
  public int getMaximumBatchSize() 
  {
    return this.P.getMaximumBatchSize();
  }

  @Override
  public boolean upgradeMaximumBatchSize(int size)
  {
    return this.P.upgradeMaximumBatchSize(size);
  }

}
