/**
 * 
 */
package sim.nn.layer;

import sim.nn.function.regularization.SDropoutFunction;
import sim.util.SStringUtil;

/**
 * La classe <b>SDropoutLayer</b> reprsente une couche de neurones appliquant uniquement la fonction dropout.
 * 
 * @author Simon Vezina
 * @since 2021-10-17
 * @version 2022-01-08
 */
public class SDropoutLayer implements SLayer {

  /**
   * ...
   */
  protected final SDropoutFunction D;
  
  /**
   * ...
   */
  protected double dropping_probability;
  
  /**
   * ...
   * 
   * @param neurons
   * @param dropping_probability
   */
  public SDropoutLayer(int neurons, double dropping_probability)
  {
    this.D = new SDropoutFunction(neurons);
    
    this.dropping_probability = dropping_probability;
  }

  /**
   * ...
   * 
   * @param l
   */
  private SDropoutLayer(SDropoutLayer l)
  {
    this.D = new SDropoutFunction(l.getOutputsSize());
    
    this.dropping_probability = l.dropping_probability;
  }
  
  @Override
  public double[] forwardPropagation(double[] input) throws IllegalArgumentException 
  {
    return this.D.drop(input, this.dropping_probability);
  }

  @Override
  public double[] backPropagation(double[] delta) throws IllegalArgumentException
  {
    return this.D.backPropagation(delta);
  }

  /**
   * Mthode pour obtenir la probabilit du dropout du neurones dans la propagation des donnes.
   * 
   * @return La probabilit du 'drop'.
   */
  public double getDropout()
  {
    return this.dropping_probability;
  }
  
  @Override
  public int getInputsSize() 
  {
    return this.D.getInputsSize();
  }

  @Override
  public int getOutputsSize() 
  {
    return this.D.getOutputsSize();
  }

  @Override
  public String getLayerName()
  {
    return SLayer.DROPOUT_LAYER;
  }
  
  @Override
  public String getDescription() 
  {
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(this.getLayerName() + " [ Dimension = " + this.getDimension() + ", D = " + this.D + " (" + this.dropping_probability +")]" + SStringUtil.END_LINE_CARACTER);
    
    return buffer.toString();
  }

  @Override
  public SLayer copyWithSameParameter() 
  {
    return new SDropoutLayer(this);
  }

  @Override
  public String toString()
  {
    return getDescription();
  }
  
}
