/**
 * 
 */
package sim.nn.layer;

import sim.exception.SConstructorException;
import sim.nn.function.filtering.SPoolingFunction;
import sim.nn.function.filtering.SPoolingFunctionBuilder;
import sim.util.SStringUtil;

/**
 * La classe <b>SPoolingLayer</b> reprsente une couche de neurones contenant une fonction de pooling.
 * 
 * @author Simon Vezina
 * @since 2021-10-18
 * @version 2022-01-08
 */
public class SPoolingLayer implements SLayer {

  /**
   * ...
   */
  protected final SPoolingFunction P;
  
  /**
   * ...
   * 
   * @param nb_inputs
   * @param inputs_depth
   * @param filter_offset
   * @param pooling_function
   * @throws SConstructorException
   */
  public SPoolingLayer(int nb_inputs, int inputs_depth, int filter_offset, String pooling_function) throws SConstructorException
  {
    // Initialisation de la fonction de pooling.
    try{
      this.P = SPoolingFunctionBuilder.build(pooling_function, nb_inputs, inputs_depth, filter_offset);
    }catch(IllegalArgumentException e){
      throw new SConstructorException("Erreur SPoolingLayer 001 : La fonction de regroupement '" + pooling_function + "' n'a pas t initialise.", e);
    } 
  }

  @Override
  public double[] forwardPropagation(double[] inputs) throws IllegalArgumentException 
  {
    return this.P.pooling(inputs);
  }

  @Override
  public double[] backPropagation(double[] delta) throws IllegalArgumentException 
  {
    return this.P.backPropagation(delta);
  }
  
  @Override
  public int getInputsSize() 
  {
    return this.P.getInputsSize();
  }

  @Override
  public int getOutputsSize() 
  {
    return this.P.getOutputsSize();
  }

  @Override
  public int getInputsDepth() 
  {
    return this.P.getInputsDepth();
  }
  
  @Override
  public int getOutputsDepth() 
  {
    return this.P.getOutputsDepth();
  }
  
  /**
   * Mthode pour obtenir la taille du filtre carr de la fonction de regroupement.
   * 
   * @return La taille du filtre.
   */
  public int getFilterOffset()
  {
    return this.P.getFilterOffset();
  }
  
  /**
   * Mthode pour obtenir le nom de la fonction de regroupement de la couche.
   * 
   * @return Le nom de la fonction de regroupement.
   */
  public String getPoolingFunctionName()
  {
    return this.P.getFunctionName();
  }
  
  @Override
  public String getDimension() 
  {
    return this.getInputsSize() + " x " + this.getOutputsSize() + " --> (" + this.P.getInputsDepth() + " x " + this.P.getInputOffset() + " x " + this.P.getInputOffset() + ") x (" + this.P.getOutputsDepth() + " x " + this.P.getOutputOffset() + " x " + this.P.getOutputOffset() + ")";
  }
  
  @Override
  public String getLayerName()
  {
    return SLayer.POOLING_LAYER;
  }
  
  @Override
  public String getDescription() 
  {
    StringBuffer buffer = new StringBuffer();
    
    buffer.append(this.getLayerName() + " [ Dimension = " + this.getDimension() + ", P = " + P + "]" + SStringUtil.END_LINE_CARACTER);
    
    return buffer.toString();
  }

  @Override
  public SLayer copyWithSameParameter() 
  {
    return new SPoolingLayer(this.getInputsSize(), this.getInputsDepth(), this.P.getFilterOffset(), this.P.getFunctionName());
  }

  @Override
  public String toString()
  {
    return getDescription();
  }
  
}
