/**
 * 
 */
package sim.nn.network;

import java.util.List;

import sim.exception.SConstructorException;
import sim.nn.layer.SBatchLayer;
import sim.nn.layer.SLayer;

/**
 * La classe <b>SBatchCustomNetwork</b> reprsente un rseau sur mesure pouvant raliser l'activation par <i>batch</i>.
 * 
 * @author Simon Vezina
 * @since 2021-11-10
 * @version 2022-01-08
 */
public class SBatchCustomNetwork implements SBatchNetwork {
  
  /**
   * ...
   */
  private final SBatchLayer[] layers;

  /**
   * ...
   * 
   * @param layers
   * @throws SConstructorException
   */
  public SBatchCustomNetwork(SBatchLayer ... layers) throws SConstructorException
  {
    if(layers.length < 1)
      throw new SConstructorException("Erreur SBatchCustomNetwork 001 : Le nombre de couche est gal  " + layers.length + " ce qui est infrieur  1.");
       
    this.layers = layers;
    
    // Vrifier si la squence de couche est valide.
    if(!isValidLayersSequence())
      throw new SConstructorException("Erreur SBatchCustomNetwork 002 : La squence des couches est invalide, car " + getDimension() + ".");  
  }
  
  /**
   * ...
   * 
   * @param layer_list
   */
  public SBatchCustomNetwork(List<SBatchLayer> layer_list)
  {
    if(layer_list.size() < 1)
      throw new SConstructorException("Erreur SBatchCustomNetwork 003 : Le nombre de couche est gal  " + layer_list.size() + " ce qui est infrieur  1.");
    
    this.layers = new SBatchLayer[layer_list.size()];
    
    for(int i = 0; i < this.layers.length; i++)
      this.layers[i] = layer_list.get(i);
    
    // Vrifier si la squence de couche est valide.
    if(!isValidLayersSequence())
      throw new SConstructorException("Erreur SBatchCustomNetwork 004 : La squence des couches est invalide, car " + getDimension() + ".");  
  }
  
  @Override
  public SLayer[] getLayers() 
  {
    return layers;
  }

  @Override
  public SNetwork copyWithSameParameter()
  {
    SBatchLayer[] copy_layers = new SBatchLayer[this.getNbLayers()];
    
    for(int i = 0 ; i < copy_layers.length; i++)
      copy_layers[i] = (SBatchLayer) this.layers[i].copyWithSameParameter();
    
    return new SBatchCustomNetwork(copy_layers);
  }

  @Override
  public String getNetworkName()
  {
    return SNetwork.CUSTOM_BATCH_NETWORK;
  }
  
}
