/**
 * 
 */
package sim.nn.network;

import java.util.ArrayList;
import java.util.List;

import sim.exception.SNoImplementationException;
import sim.nn.layer.SBatchLayer;

/**
 * La classe <b>SBatchNetwork</b> reprsente un rseau pouvant raliser l'activation et la propagation de l'erreur par <b>batch</i>.
 * Ce type de rseau peut galement utiliser des fonctions reliant plusieurs vecteur d'entre entre eux comme la fonction de normalisation <i>batch-normalization</i>.
 * 
 * @author Simon Vezina
 * @since 2018-09-15
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public interface SBatchNetwork extends SNetwork {

  @Override
  public default double[] forwardPropagation(double[] inputs) throws IllegalArgumentException
  {
    List<double[]> list = new ArrayList<double[]>();
    
    list.add(inputs);
    
    return batchForwardPropagation(1, list).get(0);
  }
  
  @Override
  public default double[] backPropagation(double[] delta) throws IllegalArgumentException
  {
    List<double[]> list = new ArrayList<double[]>();
    
    list.add(delta);
    
    return batchBackPropagation(1, list).get(0);
  }
  
  @Override
  public default List<double[]> batchForwardPropagation(int b, List<double[]> batch_inputs) throws IllegalArgumentException
  {
    // Vrifier que la taille de la batch est de taille infrieur  la taille de la liste.
    if(b > batch_inputs.size())
      throw new IllegalArgumentException("Erreur SBatchNetwork 003 : La taille de la batch ne peut pas tre suprieur  la taille de la liste d'entre.");
    
    throw new SNoImplementationException();
    
  }
  
  @Override
  public default List<double[]> batchBackPropagation(int b, List<double[]> batch_delta) throws IllegalArgumentException
  {
    // Vrifier que la taille de la batch  propager est gale  la taille de la batch qui a t active (car la taille de la batch du rseau est dfini ainsi).    
    if(b != this.getBatchSize())
      throw new IllegalArgumentException("Erreur SBatchNetwork 004 : La taille de la batch en propagation d'erreur (" + b + ") n'est pas identique  la taille de la batch en activation (" + this.getBatchSize() + ").");
    
    throw new SNoImplementationException();
    
  }
      
  @Override
  public default int getBatchSize()
  {
    return ((SBatchLayer[])this.getLayers())[0].getBatchSize();
  }
  
  @Override
  public default boolean isBatchNetwork()
  {
    return true;
  }
  
}
