/**
 * 
 */
package sim.nn.network;

import java.util.List;

import sim.exception.SConstructorException;
import sim.nn.layer.SLayer;

/**
 * La classe <b>SCustomNetwork</b> reprsente un rseau construit  l'aide d'un choix de couche fait sur mesure.
 *  
 * @author Simon Vezina
 * @since 2020-01-12
 * @version 2022-01-06
 */
public class SCustomNetwork implements SNetwork {

  /**
   * ...
   */
  private final SLayer[] layers;

  /**
   * ...
   * 
   * @param layers
   * @throws SConstructorException
   */
  public SCustomNetwork(SLayer ... layers) throws SConstructorException
  {
    if(layers.length < 1)
      throw new SConstructorException("Erreur SCustomNetwork 001 : Le nombre de couche est gal  " + layers.length + " ce qui est infrieur  1.");
       
    this.layers = layers;
    
    // Vrifier si la squence de couche est valide.
    if(!isValidLayersSequence())
      throw new SConstructorException("Erreur SCustomNetwork 002 : La squence des couches est invalide, car " + getDimension() + ".");  
  }
  
  /**
   * ...
   * 
   * @param layer_list
   */
  public SCustomNetwork(List<SLayer> layer_list)
  {
    if(layer_list.size() < 1)
      throw new SConstructorException("Erreur SCustomNetwork 003 : Le nombre de couche est gal  " + layer_list.size() + " ce qui est infrieur  1.");
    
    this.layers = new SLayer[layer_list.size()];
    
    for(int i = 0; i < this.layers.length; i++)
      this.layers[i] = layer_list.get(i);
    
    // Vrifier si la squence de couche est valide.
    if(!isValidLayersSequence())
      throw new SConstructorException("Erreur SCustomNetwork 004 : La squence des couches est invalide, car " + getDimension() + ".");  
  }
  
  @Override
  public SLayer[] getLayers() 
  {
    return layers;
  }

  @Override
  public String getNetworkName()
  {
    return SNetwork.CUSTOM_NETWORK;
  }
  
  @Override
  public SNetwork copyWithSameParameter()
  {
    SLayer[] copy_layers = new SLayer[this.getNbLayers()];
    
    for(int i = 0 ; i < copy_layers.length; i++)
      copy_layers[i] = this.layers[i].copyWithSameParameter();
    
    return new SCustomNetwork(copy_layers);
  }

}
