/**
 * 
 */
package sim.nn.parameter;

import java.util.Arrays;

import sim.nn.gradient.SGradientLayer;

/**
 * La classe <b>SBias</b> reprsente les biais d'une couche de neurones.
 * 
 * @author Simon Vezina
 * @since 2018-07-29
 * @version 2021-11-22
 */
public class SBias extends SArrayParameter {

  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur d'un vecteur de biais.
   * 
   * @param neurons Le nombre de neurones dans le vecteur de biais.
   */
  public SBias(int neurons)
  {
    this(neurons, true);  
  }

  /**
   * Constructeur d'un vecteur de biais associ  une couche de neurones.
   * 
   * @param neurons Le nombre de neurones en sortie de la couche.
   * @param random_value S'il faut dterminer des valeurs alatoires aux biais. Si faux, une valeur de zro sera accorde.
   */
  public SBias(int neurons, boolean random_value)
  {
    super(neurons, random_value);
  }
  
  /**
   * Constructeur d'un vecteur de biais  partir d'un tableau de donnes.
   * 
   * @param vector Le vecteur dcrivant le biais.
   */
  public SBias(double[] vector)
  {
    super(vector);
  }
  
  @Override
  public SBias clone()
  {
    SBias b = new SBias(this.array.clone());
    b.setTrainable(this.isTrainable());
    
    return b;
  }
  
  @Override
  public void add(SParameter parameter) throws IllegalArgumentException
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SBias))
      throw new  IllegalArgumentException("Erreur SBias 001 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    addParameter((SBias)parameter);
  }
  
  @Override
  public void multiply(SParameter parameter) throws IllegalArgumentException
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SBias))
      throw new  IllegalArgumentException("Erreur SBias 002 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    multiplyParameter((SBias) parameter);
  }
  
  @Override
  public void learn(SGradientLayer gradient, double learning_rate) 
  {
    // Effectuer l'entranement si le gradient contient un accs au poids.
    if(!(gradient instanceof SContainBias))
      throw new IllegalArgumentException("Erreur SBias 003 : Le gradient n'est pas du bon type (" + gradient.getClass().getName() + ").");
    
    learnParameter(((SContainBias)gradient).getBias(), learning_rate);
  }
    
  @Override
  public String toString()
  {
    return "SBias [bias=" + Arrays.toString(this.array) + "]";
  }
  
}
