/**
 * 
 */
package sim.nn.parameter;

import java.util.Arrays;

import sim.nn.gradient.SGradientLayer;

/**
 * La classe <b>SScale</b> reprsente le paramtre de scale dans une couche d'un rseau de neurones.
 * 
 * @author Simon Vezina
 * @since 2018-09-15
 * @version 2021-11-24
 */
public class SScale extends SArrayParameter {

  /**
   * ...
   * 
   * @param nb_output
   */
  public SScale(int nb_output)
  {
    this(nb_output, true);  
  }

  /**
   * ...
   * 
   * @param nb_output
   * @param random_value
   */
  public SScale(int nb_output, boolean random_value)
  {
    super(nb_output, random_value);  
  }
  
  /**
   * ...
   * 
   * @param vector
   */
  public SScale(double[] vector)
  {
    super(vector);
  }
  
  @Override
  public SScale clone()
  {
    SScale s = new SScale(this.array.clone());
    s.setTrainable(this.isTrainable());
    
    return s;
  }
  
  @Override
  public void add(SParameter parameter) throws IllegalArgumentException 
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SScale))
      throw new  IllegalArgumentException("Erreur SScale 001 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    addParameter((SScale) parameter);
  }

  @Override
  public void multiply(SParameter parameter) throws IllegalArgumentException
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SScale))
      throw new  IllegalArgumentException("Erreur SScale 002 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    multiplyParameter((SScale) parameter);
  }
   
  @Override
  public void learn(SGradientLayer gradient, double learning_rate) throws IllegalArgumentException 
  {
    // Effectuer l'entranement si le gradient contient un accs au poids.
    if(!(gradient instanceof SContainScale))
      throw new IllegalArgumentException("Erreur SScale 003 : Le gradient n'est pas du bon type (" + gradient.getClass().getName() + ").");
    
    learnParameter(((SContainScale)gradient).getScale(), learning_rate);
  }
  
  @Override
  public String toString()
  {
    return "SScale [scale=" + Arrays.toString(this.array) + "]";
  }
  
}
