/**
 * 
 */
package sim.nn.parameter;

import java.util.Arrays;

import sim.nn.gradient.SGradientLayer;

/**
 * La classe <b>SShift</b> reprsente le paramtre de shift dans une couche d'un rseau de neurones.
 *
 * @author Simon Vezina
 * @since 2018-09-15
 * @version 2021-11-24
 */
public class SShift extends SArrayParameter {

  /**
   * ...
   * 
   * @param nb_output
   */
  public SShift(int nb_output)
  {
    this(nb_output, true); 
  }

  /**
   * ...
   * 
   * @param nb_output
   * @param random_value
   */
  public SShift(int nb_output, boolean random_value)
  {
    super(nb_output, random_value);  
  }
  
  /**
   * ...
   * 
   * @param vector
   */
  public SShift(double[] vector)
  {
    super(vector);
  }
  
  @Override
  public SShift clone()
  {
    SShift s = new SShift(this.array.clone());
    s.setTrainable(this.isTrainable());
    
    return s;
  }
  
  @Override
  public void add(SParameter parameter) throws IllegalArgumentException 
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SShift))
      throw new  IllegalArgumentException("Erreur SShift 003 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    addParameter((SShift) parameter);
  }

  @Override
  public void multiply(SParameter parameter) throws IllegalArgumentException 
  {
    // Vrification du type de paramtre.
    if(!(parameter instanceof SShift))
      throw new  IllegalArgumentException("Erreur SShift 004 : Le paramtre n'est pas du bon type (" + parameter.getClass().getName() + ")."); 
    
    multiplyParameter((SShift) parameter);
  }
    
  @Override
  public void learn(SGradientLayer gradient, double learning_rate) throws IllegalArgumentException
  {
    // Effectuer l'entranement si le gradient contient un accs au poids.
    if(!(gradient instanceof SContainShift))
      throw new IllegalArgumentException("Erreur SShift 002 : Le gradient n'est pas du bon type (" + gradient.getClass().getName() + ").");
    
    learnParameter(((SContainShift)gradient).getShift(), learning_rate);
  }
  
  @Override
  public String toString()
  {
    return "SShift [shift=" + Arrays.toString(this.array) + "]";
  }
}
