/**
 * 
 */
package sim.nn.reader;

import java.util.ArrayList;
import java.util.List;

import sim.nn.SNNData;

/**
 * La classe <b>SAbstractNeuralDataReader</b> reprsente un lecteur de donne pour rseau de neurones abstrait.
 * 
 * @author Simon Vezina
 * @since 2019-12-08
 * @version 2021-10-15
 */
public abstract class SAbstractNeuralDataReader implements SNeuralDataReader {

  /**
   * La variable <b>training_list</b> reprsente la liste des donnes d'entranement.
   */
  protected final List<SNNData> training_list;
  
  /**
   * La variable <b>testing_list</b> reprsente la liste des donnes de test.
   */
  protected final List<SNNData> testing_list;
  
  /**
   * Constructeur d'un lecteur de donne abstrait pour rseau de neurones.
   * Ce constructeur a pour but de faire l'allocation de mmoire aux listes contenant les donnes.
   */
  public SAbstractNeuralDataReader()
  {
    this.training_list = new ArrayList<SNNData>();
    this.testing_list = new ArrayList<SNNData>();
  }

  @Override
  public List<SNNData> getAllTrainingData()
  {
    return this.training_list;
  }
  
  @Override
  public List<SNNData> getAllTestingData()
  {
    return this.testing_list;
  }
  
  @Override
  public int getDataSize() throws IllegalStateException
  {
    if(this.training_list.size() == 0)
      throw new IllegalStateException("Erreur SAbstractNeuralDataREader 001 : Puisqu'aucune donne n'a t lu, la taille des donnes est indtermine.");
      
    return this.training_list.get(0).getDataSize();
  }

  @Override
  public int getClassificationSize() 
  {
    if(this.training_list.size() == 0)
      return 0;
    
    return this.training_list.get(0).getExpectedSize();
  }
  
  @Override
  public void clear()
  {
    this.training_list.clear();
    this.testing_list.clear();
  }
  
  /**
   * Mthode pour construire une liste de vecteur d'activation attendu pour l'ensemble des classifications des donnes.
   * 
   * @param size Le nombre de classification disponible.
   * @return Une liste de vecteur d'activation attendu.
   */
  protected List<double[]> buildExpectedValue(int size)
  {
    List<double[]> list = new ArrayList<double[]>();
    
    // Itrer sur l'ensemble de la classification.
    for(int i = 0; i < size; i++)
    {
      // Allocation de la mmoire du vecteur attendue.
      double[] expected_vector = new double[size];
      
      // Affectation de sa valeur : 1  son indice de classification et 0 partour.
      expected_vector[i] = 1.0;   
      
      list.add(expected_vector);
    }
    
    return list;
  }
  
}
