/**
 * 
 */
package sim.nn.reader;

import sim.util.SStringUtil;

/**
 * ...
 * 
 * @author Simon Vezina
 * @since 2019-05-24
 * @version 2019-06-03
 */
public abstract class SOLDAbstractNeuralImageDataRW {

  /**
   * La constante <b>DEFAULT_BLACK_VALUE</b> reprsente la valeur numrique attribuable  la couleur noire tant {@value}.
   */
  private final static double DEFAULT_BLACK_VALUE = 0.0;
  
  /**
   * La constante <b>DEFAULT_WHITE_VALUE</b> reprsente la valeur numrique attribuable  la couleur blanche tant {@value}.
   */
  private final static double DEFAULT_WHITE_VALUE = 1.0;
    
  protected double black_value;
  protected double white_value;
  protected double red_value;
  protected double green_value;
  protected double blue_value;
  
  protected final boolean is_colored;
  protected final String data_directory;
  protected final String[] classifications;
  protected final double[][] expected_vector;
  
  /**
   * ...
   * 
   * @param data_directory
   * @param classification
   * @param colored_image
   */
  public SOLDAbstractNeuralImageDataRW(String data_directory, String[] classifications, boolean colored_image) 
  {
    this.is_colored = colored_image;
    this.data_directory = data_directory;
    this.classifications = classifications;
    this.expected_vector = new double[classifications.length][];
    
    this.black_value = DEFAULT_BLACK_VALUE;
    this.white_value = DEFAULT_WHITE_VALUE;
    this.red_value = DEFAULT_WHITE_VALUE;
    this.green_value = DEFAULT_WHITE_VALUE;
    this.blue_value = DEFAULT_WHITE_VALUE;
            
    buildExpectedValue();
  }

  /**
   * ...
   * 
   * @return
   */
  public double[][] getAllExpectedVector()
  {
    return this.expected_vector;
  }
  
  /**
   * ...
   * 
   * @return
   * @throws IllegalArgumentException
   */
  public double[] getExpectedVector(String classification) throws IllegalArgumentException
  {
    for(int i = 0; i < this.classifications.length; i++)
      if(classifications[i].equals(classification))
        return this.expected_vector[i];
    
    throw new IllegalArgumentException("Erreur SAbstractNeuralImageDataRW 001 : La classification '" + classification + "' n'est pas reconnu. Les classifications valides sont les suivantes : " + SStringUtil.join(this.classifications,  ", "));
  }
  
  /**
   * ...
   * 
   * @return
   */
  public String[] getAllClassifications()
  {
    return this.classifications;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setWhiteNeuronValue(double value)
  {
    this.white_value = value;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setBlackNeuronValue(double value)
  {
    this.black_value = value;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setRedNeuronValue(double value)
  {
    this.red_value = value;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setGreenNeuronValue(double value)
  {
    this.green_value = value;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setBlueNeuronValue(double value)
  {
    this.blue_value = value;
  }
  
  /**
   * Mthode pour dterminer si la classification est reconnue.
   * 
   * @param classification La classification  dterminer.
   * @return <b>true</b> si la classification est reconnue et <b>false</b> sinon.
   */
  public boolean isValidClassification(String classification)
  {
    for(int i = 0; i < this.classifications.length; i++)
      if(classifications[i].equals(classification))
        return true;
    
    return false;
  }
  
  /**
   * Mthode pour construire les vecteurs des valeurs attendues de nos donnes.
   */
  private void buildExpectedValue()
  {
    // Itrer sur l'ensemble de la classification.
    for(int i = 0; i < classifications.length; i++)
    {
      // Allocation de la mmoire du vecteur attendue.
      expected_vector[i] = new double[classifications.length];
      
      // Affectation de sa valeur : 1  son indice de classification et 0 partour.
      expected_vector[i][i] = 1.0;                               
    }
  }
  
}
