/**
 * 
 */
package sim.nn.reader;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import sim.exception.SNoImplementationException;
import sim.graphics.SColor;
import sim.math.SArrayMatrix;
import sim.nn.SNNData;
import sim.util.SBrowser;
import sim.util.SStringUtil;

/**
 * La classe <b>SNeuralImageDataWriter</b> permet l'criture de donnes en fichier image.
 * 
 * @author Simon Vezina
 * @since 2019-05-23
 * @version 2019-12-06
 */
public class SOLDNeuralImageDataWriter extends SOLDAbstractNeuralImageDataRW{

  /**
   * ...
   */
  private final String DEFAULT_IMAGE_EXTENSION = "png";
  
  /**
   * ...
   */
  private final List<SNNData> training_list; 
  
  /**
   * ...
   * 
   * @param data_directory
   * @param classification
   * @param colored_image
   */
  public SOLDNeuralImageDataWriter(String data_directory, String[] classification, boolean colored_image) 
  {
    super(data_directory, classification, colored_image);
    
    training_list = new ArrayList<SNNData>();
  }

  /**
   * ...
   * 
   * @param obj
   * @param vector
   * @param classification
   * @throws IllegalArgumentException
   */
  public void addTrainingData(Object obj, double[] vector, String classification) throws IllegalArgumentException
  {
    if(!isValidClassification(classification))
      throw new IllegalArgumentException("Erreur SNeuralImageDataWriter 002 : La classification '" + classification + "' n'est pas reconnu. Les classifications valides sont les suivantes : " + SStringUtil.join(this.classifications,  ", "));
    
    double[] expected = getExpectedVector(classification);
    
    SNNData data = new SNNData(obj, vector, expected, classifications);
    
    this.training_list.add(data);
  }
  
  /**
   * Mthode pour crire l'ensembles des donnes en fichier image.
   * 
   * @throws IOException Si une erreur de type I/O est survenue.
   */
  public void writeTrainingData() throws IOException
  {
    // Cration d'un browser.
    SBrowser browser = new SBrowser();
    
    // Cration au besoin des rpertoire contenant les images.
    browser.createDirectory(this.data_directory);
    browser.goTo(this.data_directory);
    
    for(int i = 0; i < this.classifications.length; i++)
      browser.createDirectory(this.classifications[i]);
    
    // Cration des images.
    for(SNNData data : this.training_list)
    {
      // Dplacer le browser dans le rpertoire de la classification de la donne.
      browser.forward(data.getExpectedClassification());
      
      // Gnrer l'image de la donne.
      BufferedImage buffer = arrayToImage(data.getVector());
    
      // crire l'image avec le browser.
      browser.writeImageFile(data.getObj().toString(), DEFAULT_IMAGE_EXTENSION, buffer);
            
      // Reculer le browser.
      browser.back();
    }
  }
  
  /**
   * Mthode pour gnrer une image noire et blanche  partir d'un tableau de valeur.
   * 
   * @param array
   * @return
   */
  public BufferedImage arrayToImage(double[] array)
  {
    SArrayMatrix matrix = new SArrayMatrix(array);
    
    BufferedImage image = new BufferedImage(matrix.nbColumns(), matrix.nbLines(), BufferedImage.TYPE_INT_ARGB);
    
    // La valeur delta fait l'cart entre la correspondance des "couleur noir" et "couleur blanche".
    final double delta = 2.0*(this.white_value - this.black_value);
        
    // La base de la correspondance  des "couleur noir" et "couleur blanche".
    final double max = this.white_value;
    
    // Itrer sur l'ensemble des pixels de l'image.
    for(int i = 0; i < matrix.nbLines(); i++)
      for(int j = 0; j < matrix.nbColumns(); j++)
      {
        double value = (matrix.get(i, j) + max) / delta;
        
        // Normalisation de la valeur entre [0, 1].
        if(value < 0.0)
          value = 0.0;
        
        if(value > 1.0)
          value = 1.0;
        
        // Construction de la couleur.
        SColor color = new SColor(value, value, value);
        
        // Affectation de la couleur dans l'image.
        image.setRGB(i, j, color.normalizeColor().getRGB());
      }
    
    return image;
  }
  
  /**
   * ...
   * 
   * @param depth_array
   * @return
   */
  public BufferedImage depthArrayToImage(List<double[]> depth_array)
  {
    throw new SNoImplementationException();
  }
  
}
