/**
 * 
 */
package sim.nn.trainer;

import java.util.ArrayList;
import java.util.List;

import sim.exception.SConstructorException;
import sim.nn.SNNData;
import sim.nn.function.error.SErrorFunction;
import sim.nn.function.error.SErrorFunctionBuilder;
import sim.nn.network.SNetwork;

/**
 * La classe <b>SErrorsComputer</b> reprsente un calculateur d'erreur sur l'activation d'une donne par un rseau. 
 * 
 * Cette classe a pour objectif d'tre utilis en parallle (multi-tche).
 * 
 * @author Simon Vezina
 * @since 2019-09-05
 * @version 2021-09-12
 */
public class SErrorsComputer implements Runnable {

  /**
   * ...
   */
  private final SNetwork network;
  
  /**
   * ...
   */
  private final SErrorFunction error_function;
  
  /**
   * ...
   */
  private final List<SNNData> data_list;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur d'un calculateur d'erreur pour l'activation d'un rseau de neurones.
   * 
   * @param network Le rseau.
   * @param error_function_name Le nom de la fonction d'erreur utilis dans le calcul de l'erreur.
   */
  public SErrorsComputer(SNetwork network, String error_function_name)
  {
    // Afin de supporter le multi-processeur, je dois faire une copie de ce rseau
    // puisque l'activation ncessite de l'accs en mmoire dans le rseau.
    this.network = network.copyWithSameParameter();
        
    this.data_list = new ArrayList<SNNData>();
    
    try {
      this.error_function = SErrorFunctionBuilder.build(error_function_name, this.network.getOutputsSize());
    }catch(IllegalArgumentException e) {
      throw new SConstructorException("Erreur SErrorsComputer 001 : Une erreur lors de la cration de la fonction d'erreur est survenue.", e);
    }
  }

  /**
   * Mthode pour ajouter une donne  activer par le rseau.
   * 
   * @param data La donne.
   */
  public void add(SNNData data)
  {
    this.data_list.add(data);
  }
  
  /**
   * Mthode pour vider la liste des donnes  activer par le rseau.
   */
  public void clear()
  {
    this.data_list.clear();
  }
  
  @Override
  public void run()
  {
    for(SNNData data : data_list)
      STrainingAlgorithm.computeErrors(this.network, this.error_function, data);
  }

}
