/**
 * 
 */
package sim.nn.trainer;

import sim.exception.SConstructorException;
import sim.nn.gradient.SErrorFunctionGradient;
import sim.nn.network.SNetwork;

/**
 * La classe <b>SMomentumTrainer</b>
 * 
 * @author Simon Vezina
 * @since 2019-09-13
 * @version 2022-01-01
 */
public class SMomentumTrainer extends SFastTrainer {

  /**
   * La constante <b>DEFAULT_MOMENTUM</b> reprsente l'inertie par dfaut tant gal  {@value}.
   */
  protected final static double DEFAULT_MOMENTUM = 0.9;
  
  /**
   * La variable <b>momentum_gradient</b> reprsente le gradient avec inertie associ  la fonction d'erreur.
   */
  protected final SErrorFunctionGradient momentum_gradient;
  
  /**
   * La variable <b>momentum</b> reprsente l'inertie du gradient. 
   */
  protected double momentum;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * @param network
   * @param error_function
   * @throws SConstructorException
   */
  public SMomentumTrainer(SNetwork network, String error_function) throws SConstructorException 
  {
    super(network, error_function);
    
    // Constructeur du gradient avec inertie.
    this.momentum_gradient = new SErrorFunctionGradient(network);
    
    // Dfinir l'inertie.
    this.momentum = DEFAULT_MOMENTUM;
  }

  @Override
  public void train(int mini_batch)
  {
    STrainingAlgorithm.parallelTrainingWithMomentum(this.network, this.gradient, this.momentum_gradient, this.computer_gradient_list, getRandomTrainingDataList(mini_batch), this.momentum, this.learning_rate);
    
    // Faire la mise  jour avec 1 entranement utilisant X donnes.
    this.updateTrainingStatistic(mini_batch);
  }
  
  @Override
  public String toString() 
  {
    return "SMomentumTrainer [error_function = " + error_function + ", learning_rate = " + learning_rate + ", momentum = " + DEFAULT_MOMENTUM + ", Processors = " + this.getProcessors() + ", Training vector = " + training_list.size() + ", Testing vector = " + testing_list.size() + "]";
  }
  
}
