/**
 * 
 */
package sim.nn.trainer;

import java.util.ArrayList;
import java.util.List;

import sim.exception.SNoImplementationException;
import sim.exception.SRuntimeException;
import sim.nn.SNNData;
import sim.nn.function.error.SBatchErrorFunction;
import sim.nn.function.error.SErrorFunction;
import sim.nn.gradient.SErrorFunctionGradient;
import sim.nn.gradient.SGradientAlgorithm;
import sim.nn.gradient.SGradientComputer;
import sim.nn.network.SBatchNetwork;
import sim.nn.network.SNetwork;

/**
 * La classe <b>STrainingAlgorithm</b> est une classe utilitaire effectuant des oprations d'entranement sur des rseaux.
 * 
 * @author Simon Vezina
 * @since 2019-09-01
 * @version 2022-09-14 (Version labo : Apprentissage par rseau de neurones)
 */
public class STrainingAlgorithm {

  /**
   * Mthode pour valuer l'erreur gnre par un rseau  partir d'une donne.
   * L'erreur et la classification calcules seront enregistres dans la structure de la donne. 
   * 
   * @param network Le rseau.
   * @param error_function La fonction d'erreur 
   * @param data La donne.
   * @return les erreurs du rseau en lien avec la donne.
   */
  public static double[] computeErrors(SNetwork network, SErrorFunction error_function, SNNData data)
  {
    throw new SNoImplementationException();
    
  }
    
  /**
   * Mthode pour rsaliser le calcul d'erreur en mode parallle.
   * 
   * @param computer_list La liste des calculateur d'ereur.
   */
  public static void paralleleComputeErrors(List<SErrorsComputer> computer_list)
  {
    // Construire la list de threads.
    List<Thread> thread_list = new ArrayList<Thread>();
    
    // Remplir la liste de theads.
    for(SErrorsComputer c : computer_list)
      thread_list.add(new Thread(c));
    
    // Partir l'ensemble des threads
    for(Thread t : thread_list)
      t.start();
    
    // Faire l'attente de la fin de toutes les threads de la liste
    for(Thread t : thread_list)
      try{
        t.join();
      }catch(InterruptedException e){
        throw new SRuntimeException("Erreur STrainingAlgorithm 001 : Un thread fut interropue spontanment.", e);
      }
  }
  
  /**
   * Mthode pour faire l'entranement du rseau  l'aide d'une donnes.
   * 
   * @param network Le rseau.
   * @param error_function La fonction d'erreur.
   * @param gradient Le gradient de la fonction d'erreur.
   * @param data La donne.
   * @param learning_rate Le taux d'apprentissage.
   */
  public static void training(SNetwork network, SErrorFunction error_function, SErrorFunctionGradient gradient, SNNData data, double learning_rate)
  {
    throw new SNoImplementationException();
    
  }
    
  /**
   * Mthode pour raliser un entranement en mode <i>batch</i>. Pour raliser ce mode d'entranement, le rseau doit ncessairement est de type <i>batch-network</i>.
   * 
   * @param batch_network Le rseau de type <i>batch</i>.
   * @param batch_error_function La fonction d'erreur de type <i>batch</i>.
   * @param gradient Le gradient de la fonction d'erreur.
   * @param data_list La liste des donnes.
   * @param learning_rate Le taux d'apprentissage.
   * @throws IllegalArgumentException Si le rseau n'est pas du bon type.
   */
  public static void batchTraining(SBatchNetwork batch_network, SBatchErrorFunction batch_error_function, SErrorFunctionGradient gradient, List<SNNData> data_list, double learning_rate) 
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * Mthode pour faire l'apprentissage du rseau  l'aide de plusieurs donnes. 
   * Ce mode d'apprentissage est de type <i>mini-batch</i>.
   * 
   * @param network Le rseau.
   * @param error_function La fonction d'erreur.
   * @param gradient Le gradient de la fonction d'erreur.
   * @param data_list La liste des donnes.
   * @param learning_rate Le taux d'apprentissage.
   */
  public static void training(SNetwork network, SErrorFunction error_function, SErrorFunctionGradient gradient, List<SNNData> data_list, double learning_rate)
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * Mthode pour raliser des entrainements en mode parallle pour <i>mini-batch</i>.
   * 
   * @param network Le rseau.
   * @param gradient Le gradient de la focntion d'Erreur.
   * @param computer_list La liste des calculateurs de gradient.
   * @param data_list La liste des donnes.
   * @param learning_rate Le taux d'apprentissage.
   */
  public static void parallelTraining(SNetwork network, SErrorFunctionGradient gradient, List<SGradientComputer> computer_list, List<SNNData> data_list, double learning_rate)
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * Mthode pour raliser des entrainements en mode parallle avec <i>momentum</i>.
   * 
   * @param network Le rseau.
   * @param gradient Le gradient de la fonction d'erreur.
   * @param momentum_gradient Le gradient de la fonctin d'erreur avec <i>momentum</i>.
   * @param computer_list La liste des calculateurs de gradient.
   * @param data_list La liste des donnes.
   * @param momentum Le facteur de momentum (entre 0 et 1, habituellement 0.9).
   * @param learning_rate Le taux d'apprentissage.
   */
  public static void parallelTrainingWithMomentum(SNetwork network, SErrorFunctionGradient gradient, SErrorFunctionGradient momentum_gradient, List<SGradientComputer> computer_list, List<SNNData> data_list, double momentum, double learning_rate)
  {
    throw new SNoImplementationException();
    
  }
  
}
