package sim.nn.wdbc;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import sim.exception.SConstructorException;
import sim.exception.SNoImplementationException;
import sim.nn.SNNData;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SLog;
import sim.util.SReadable;
import sim.util.SReader;


/**
 * La classe <b>SWDBCData</b> reprsente une base de donnes correspondant  la collection d'image WDBC (https://archive.ics.uci.edu/ml/datasets/Breast+Cancer+Wisconsin+(Diagnostic)).
 * 
 * @author Simon Vzina
 * @since 2018-03-01
 * @version 2018-03-23
 */
public class SWDBCData implements SReadable {

  /**
   * ...
   */
  private final List<SPatient> patient_list;
  
  /**
   * ...
   * 
   * @param file_name
   * @throws SConstructorException Une erreur de construction est survenue.
   */
  public SWDBCData(String file_name) throws SConstructorException
  {
    patient_list = new ArrayList<SPatient>();
    
    try{
      
      SReader reader = new SReader();
      reader.read(file_name, this);
      
    }catch(FileNotFoundException e){
      throw new SConstructorException("Erreur SWDBCData 001 : Un fichier n'a pas t trouv.", e);
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SWDBCData 002 : Une erreur d'initialisation est survenue.", e);
    }catch(IOException e){
      throw new SConstructorException("Erreur SWDBCData 003 : Une erreur de type I/O est survenue.", e);
    }
  }
  
  public List<SPatient> getPatientList()
  {
    return patient_list;
  }
  
  /**
   * Mthode pour obtenir une liste de donne pour rseau de neurones.
   * 
   * @return Une liste de donne pour rseau de neurones.
   */
  public List<SNNData> getDataList()
  {
    List<SNNData> data_list = new ArrayList<SNNData>();
    
    for(SPatient p : patient_list)
      data_list.add(new SNNData(p.toString(), p.getVector(), p.getValue()));
    
    return data_list;
  }
  
  @Override
  public void read(SBufferedReader sbr) throws IOException, SInitializationException 
  {
    boolean end = false;
    
    do{
    
      String line = sbr.readLine();
      
      // Condition de fin de lecture.
      if(line == null)
        end = true;
      else
      {
        try{
          SPatient p = new SPatient(line);
          patient_list.add(p);
        }catch(SConstructorException e){
          SLog.logWriteLine("Le patient  la ligne '" + line + "' n'a pas t ajout  la liste des patients.");
        }
      }          
    }while(!end);  
  }

  @Override
  public String getReadableName() 
  {
    throw new SNoImplementationException();
  }

  @Override
  public String[] getReadableParameterName()
  {
    throw new SNoImplementationException();
  }
  
}
