/**
 * 
 */
package sim.readwrite;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * L'interface <b>SRW</b> permet  un objet de pouvoir tre lu et crit  partir d'un fichier texte. 
 * 
 * @author Simon Vzina
 * @since 2018-03-24
 * @version 2022-04-23
 */
public interface SRW {

  /**
   * Mthode qui permet la lecture d'un fichier de format txt  partir d'un objet de type SRW.
   * 
   * @throws SReadingException S'il y a eu une erreur lors de la lecture.
   */
  public void read() throws SReadingException;
   
  /**
   * Mthode pour crire un objet <b>SRW</b> dans un fichier texte dont le nom doit tre dtermin par l'objet lui-mme.
   * 
   * @throws IOException Si une erreur de type I/O est survenue.
   */
  public void write() throws IOException;
  
  /**
   * Mthode pour crire un objet <b>SRW</b> dans un fichier texte en utilisant un <b>BufferedWriter</b>.
   * 
   * @param bw Le buffer pour l'criture.
   * @throws IOException Si une erreur de type I/O est survenue.
   */
  public default void write(BufferedWriter bw) throws IOException
  {
    bw.write(this.getRWName());
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_END);
    bw.write(SStringUtil.END_LINE_CARACTER);
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  /**
   * Mthode permettant d'crire les information d'un objet SRW dans un fichier texte.
   * 
   * @param bw Le buffer d'criture.
   * @throws IOException Si une erreur de type I/O est survenue.
   */
  public void writeInformation(BufferedWriter bw) throws IOException;
   
  /**
   * Mthode pour obtenir le nom de l'objet implmentant l'interface <b>SReadable</b>.
   * Ce nom correspond galement au <b>mot cl</b>  rechercher lors d'une lecture avec un <b>SBufferedReader</b> pour en faire la construction de l'objet en question.
   *    
   * @return Le nom de l'objet lisible.
   */
  public String getRWName();
  
  /**
   * Mthode pour obtenir un tableau des <b>mots cls</b> permettant de dfinir les paramtres de l'objet implmentant l'interface <b>SReadable</b> lors de sa lecture.
   * 
   * @return Un tableau des <b>mots cls</b> des paramtres.
   */
  public String[] getRWParameterName();
  
}
