/**
 * 
 */
package sim.readwrite.object.nn;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.exception.SConstructorException;
import sim.math.SBufferedMatrix;
import sim.nn.layer.SBatchFullConnectedLayer;
import sim.readwrite.object.SMatrixRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * ...
 * 
 * @author Simon Vezina
 * @since 2021-09-20
 * @version 2022-03-04
 */
public class SBatchFullConnectedLayerRW extends SFullConnectedLayerRW {

  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    SKeyWordDecoder.KW_SCALE, SKeyWordDecoder.KW_SHIFT 
  };
  
  /**
   * ...
   */
  private final static double[] DEFAULT_SCALE = { };
  
  /**
   * ...
   */
  private final static double[] DEFAULT_SHIFT = { };
  
  //-------------
  // VARIABLES //
  //-------------
  
  /**
   * La variable <b>scale</b> reprsente les donnes correspondant au scale de la couche.
   */
  private double[] scale;
  
  /**
   * La variable <b>shift</b> reprsente les donnes correspondant au shift de la couche.
   */
  private double[] shift;
  
  /**
   * 
   */
  public SBatchFullConnectedLayerRW()
  {
    super();
    
    this.scale = DEFAULT_SCALE;
    this.shift = DEFAULT_SHIFT;
  }

  /**
   * @param file_name
   */
  public SBatchFullConnectedLayerRW(String file_name) 
  {
    super(file_name);

    this.scale = DEFAULT_SCALE;
    this.shift = DEFAULT_SHIFT;
  }

  /**
   * ...
   * 
   * @param file_name
   * @param aggregation_function
   * @param activation_function
   * @param matrix
   * @param bias
   * @param activation_parameters
   * @param scale
   * @param shift
   * @throws SConstructorException
   */
  public SBatchFullConnectedLayerRW(String file_name, String aggregation_function, String activation_function, SMatrixRW matrix, double[] bias, SMatrixRW activation_parameters, double[] scale, double[] shift) throws SConstructorException 
  {
    super(file_name, aggregation_function, activation_function, matrix, bias, activation_parameters);
   
    this.scale = scale;
    this.shift = shift;
    
    try{
      initialization();
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SBatchFullConnectedLayerRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }

  /**
   * @param sbr
   * @throws IOException
   * @throws SConstructorException
   */
  public SBatchFullConnectedLayerRW(SBufferedReader sbr) throws IOException, SConstructorException 
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SBatchFullConnectedLayerRW 002 : Une erreur d'initialisation est survenue.", e);
    }
  }

  /**
   * ...
   * 
   * @return
   */
  public double[] getScale()
  {
    return this.scale;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double[] getShift()
  {
    return this.shift;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SBatchFullConnectedLayer toBatchFullConnectedLayer()
  {
    return new SBatchFullConnectedLayer(
        new SBufferedMatrix(this.getWeights().getMatrix()), 
        this.getBias().clone(), 
        new SBufferedMatrix(this.getActivationParameters().getMatrix()),
        this.getScale().clone(), 
        this.getShift().clone(), 
        this.getAggregationFunctionName(), 
        this.getActivationFunctionName()
        );
  }
  
  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    // criture des autres paramtres.
    super.writeInformation(bw);
 
    // criture du SCALE.
    bw.write(SKeyWordDecoder.KW_SCALE);
    bw.write("\t");
    
    // crire tous les composants de la ligne.
    for(int i = 0; i < this.scale.length; i++)
    {
      bw.write(Double.toString(scale[i]));
      bw.write("\t");
    }
    bw.write(SStringUtil.END_LINE_CARACTER);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture du SHIFT.
    bw.write(SKeyWordDecoder.KW_SHIFT);
    bw.write("\t");
    
    // crire tous les composants de la ligne.
    for(int i = 0; i < this.shift.length; i++)
    {
      bw.write(Double.toString(shift[i]));
      bw.write("\t");
    }
    bw.write(SStringUtil.END_LINE_CARACTER);   
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_BATCH_FULL_CONNECTED_LAYER;
  }

  @Override
  protected void initialization() throws SInitializationException
  {
    super.initialization();
    
    // Initialisation si la lecture/affectation a t ralise.
    
    if(this.scale != null)
      if(this.scale.length != this.getBias().length)
        throw new SInitializationException("Erreur SBathFullConnectedLayerRW 003 : La couche possde '" + this.getBias().length + " biais et le scale est de '" + this.scale.length + "' lments ce qui n'est pas gaux."); 
  
    if(this.shift != null)
      if(this.shift.length != this.getBias().length)
        throw new SInitializationException("Erreur SBathFullConnectedLayerRW 004 : La couche possde '" + this.getBias().length + " biais et le shift est de '" + this.shift.length + "' lments ce qui n'est pas gaux.");
  }

  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_SCALE: this.scale = readDoubleArray(remaining_line, SKeyWordDecoder.KW_SCALE); return true;
      case SKeyWordDecoder.CODE_SHIFT: this.shift = readDoubleArray(remaining_line, SKeyWordDecoder.KW_SHIFT); return true;
      
      default : return super.read(sbr, code, remaining_line);
    }
  }

  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
  
}
