/**
 * 
 */
package sim.readwrite.object.nn;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import sim.exception.SConstructorException;
import sim.math.SArrayTensor;
import sim.math.SBufferedMatrix;
import sim.nn.function.SNeuralMath;
import sim.nn.layer.SConvolutionLayer;
import sim.nn.parameter.SActivationParameters;
import sim.nn.parameter.SBias;
import sim.nn.parameter.SFiltersWeights;
import sim.readwrite.object.SMatrixRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * ...
 * 
 * @author Simon Vezina
 * @since 2021-09-17
 * @version 2021-12-31
 */
public class SConvolutionLayerRW extends SLayerRW {
  
  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = { 
      SKeyWordDecoder.KW_FILTERS, 
      SKeyWordDecoder.KW_BIAS, 
      SKeyWordDecoder.KW_ACTIVATION_PARAMETERS,
      SKeyWordDecoder.KW_ACTIVATION_FUNCTION,  
      SKeyWordDecoder.KW_INPUT_SIZE,
      SKeyWordDecoder.KW_STRIDE, 
      SKeyWordDecoder.KW_PADDING 
      };
    
  private static final int DEFAULT_INPUT_SIZE = 1;
  private static final int DEFAULT_STRIDE = 1;
  private static final int DEFAULT_PADDING = 0;
  
  //-------------
  // VARIABLES //
  //-------------
  
  /**
   * La variable <b>activation_function</b> reprsente le code de rfrence du choix de la fonction d'activation.
   */
  private int activation_function;
  
  /**
   * La variable <b>matrix</b> reprsente les donnes coorrespondant au poids (weight) de la couche.
   */
  private List<SFiltersRW> filters_list;
  
  /**
   * La variable <b>bias</b> reprsente les donnes correspondant au bias de la couche.
   */
  private double[] bias;
  
  /**
   * ...
   */
  private SMatrixRW activation_parameters;
  
  /**
   * ...
   */
  private int input_size;
  
  /**
   * ...
   */
  private int stride;
  
  /**
   * ...
   */
  private int padding;
  
  /**
   * ...
   */
  public SConvolutionLayerRW()
  {
    super();
    
    this.activation_function = 0;
    this.filters_list = new ArrayList<SFiltersRW>();
    this.bias = DEFAULT_BIAS;
    this.activation_parameters = DEFAULT_MATRIX;
    
    this.input_size = DEFAULT_INPUT_SIZE;
    this.stride = DEFAULT_STRIDE;
    this.padding = DEFAULT_PADDING;
  }
  
  /**
   * ....
   * 
   * @param file_name
   */
  public SConvolutionLayerRW(String file_name)
  {
    super(file_name);
    
    this.activation_function = 0;
    this.filters_list = new ArrayList<SFiltersRW>();
    this.bias = DEFAULT_BIAS;
    this.activation_parameters = DEFAULT_MATRIX;
    
    this.input_size = DEFAULT_INPUT_SIZE;
    this.stride = DEFAULT_STRIDE;
    this.padding = DEFAULT_PADDING;
  }
  
  /**
   * ...
   * 
   * @param file_name
   * @param activation_function
   * @param W
   * @param bias
   * @param activation_paramters
   * @param input_size
   * @param stride
   * @param padding
   * @throws SConstructorException
   */
  public SConvolutionLayerRW(String file_name, String activation_function, SFiltersWeights W, double[] bias, SMatrixRW activation_paramters, int input_size, int stride, int padding) throws SConstructorException
  {
    super(file_name);
    
    // Validation du nom de la fonction d'agrgation.
    int act_index = Arrays.asList(ACTIVATION_FUNCTIONS).indexOf(activation_function);
    
    if(act_index == -1)
      throw new SConstructorException("Erreur SFiltersLayerRW 002 : La fonction d'activation '" + activation_function + "' n'est pas reconnue. Les fonctions admissibles sont les suivantes : " + SStringUtil.join(ACTIVATION_FUNCTIONS, ", "));
    
        
    this.activation_function = act_index;
    
    // Obtenir la liste des filtres.
    // Le nombre sera ga  la profondeur de sortie.
    List<SArrayTensor> tensor_list = W.getTensorList();
    
    this.filters_list = new ArrayList<SFiltersRW>();
    
    for(SArrayTensor t : tensor_list)
    {
      // Obtenir la sous-liste des matrices de convolution.
      // Le nombre sera gal  la profondeur d'entre.
      List<SArrayTensor> sub_tensor = t.toListLowerOrderTensor();
           
      List<SMatrixRW> l = new ArrayList<SMatrixRW>();
      
      for(SArrayTensor sub_t : sub_tensor)
        l.add(new SMatrixRW(sub_t.toSBufferedMatrix().getLineList()));
      
      this.filters_list.add(new SFiltersRW(l));
    }
    
    this.bias = bias;
    this.activation_parameters = activation_paramters;
    
    this.input_size = input_size;
    this.stride = stride;
    this.padding = padding;
    
    try{
      initialization();
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SConvolutionLayerRW 001 : Une erreur d'initialisation est survenue.", e);
    }
    
  }
  
  /**
   * 
   * @param sbr
   * @throws IOException
   * @throws SConstructorException
   */
  public SConvolutionLayerRW(SBufferedReader sbr) throws IOException, SConstructorException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SConvolutionLayerRW 002 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
 
    // critude du la taille d'entre (input size).
    bw.write(SKeyWordDecoder.KW_INPUT_SIZE);
    bw.write("\t");
    bw.write(Integer.toString(this.input_size));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture de la fonction d'activation.
    bw.write(SKeyWordDecoder.KW_ACTIVATION_FUNCTION);
    bw.write("\t\t");
    bw.write(ACTIVATION_FUNCTIONS[activation_function]);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // critude du stride.
    bw.write(SKeyWordDecoder.KW_STRIDE);
    bw.write("\t\t");
    bw.write(Integer.toString(this.stride));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture du padding.
    bw.write(SKeyWordDecoder.KW_PADDING);
    bw.write("\t\t");
    bw.write(Integer.toString(this.padding));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // crire les filtres.
    for(SFiltersRW f : filters_list)
      f.write(bw);
        
    // crire les biais.
    bw.write(SKeyWordDecoder.KW_BIAS);
    bw.write("\t");
    
    // crire tous les composants de la ligne.
    for(int i = 0; i < bias.length; i++)
    {
      bw.write(Double.toString(bias[i]));
      bw.write("\t");
    }
    bw.write(SStringUtil.END_LINE_CARACTER);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture des parametres d'activation.
    bw.write(SKeyWordDecoder.KW_ACTIVATION_PARAMETERS);
    bw.write(SStringUtil.END_LINE_CARACTER);
    activation_parameters.writeInformation(bw);
    bw.write(SKeyWordDecoder.KW_END);     // fermeture du mot cl KW_ACTIVATION_PARAMETERS.
    bw.write(SStringUtil.END_LINE_CARACTER);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_CONVOLUTION_LAYER;
  }
  
  /**
   * Mthode pour obtenir le nom de la fonction d'activation de la couche.
   * 
   * @return Le nom de la fonction d'activation.
   */
  public String getActivationFunctionName()
  {
    return ACTIVATION_FUNCTIONS[activation_function];
  }
  
  /**
   * Mthode pour obtenir le nombre de neurones en entre de la couche.
   * 
   * @return Le nombre de neurones en entre.
   */
  public int getInputSize()
  {
    return this.input_size;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getStride()
  {
    return this.stride;    
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getPadding()
  {
    return this.padding;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public List<SFiltersRW> getFilters()
  {
    return this.filters_list;    
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double[] getBias()
  {
    return this.bias;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SMatrixRW getActivationParameters()
  {
    return this.activation_parameters;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SConvolutionLayer toConvolutionLayer()
  {
    // Construire les filtres.
    List<SFiltersRW> l = this.getFilters();
    SArrayTensor[] array = new SArrayTensor[l.size()];
      
    for(int i = 0; i < l.size(); i++)
    {
      SFiltersRW filtersRW = l.get(i);
        
      List<SMatrixRW> matrixRW_list = filtersRW.getMatrix();
      List<SBufferedMatrix> matrix_list = new ArrayList<SBufferedMatrix>();
        
      for(SMatrixRW m : matrixRW_list)
        matrix_list.add(new SBufferedMatrix(m.getMatrix()));
        
      array[i] = SArrayTensor.buildWithMatrixList(matrix_list);
    }
                
    SFiltersWeights W = new SFiltersWeights(array);
      
    return new SConvolutionLayer(
        W, 
        new SBias(this.getBias()), 
        new SActivationParameters(new SBufferedMatrix(this.getActivationParameters().getMatrix())),
        this.getInputSize(), 
        this.getPadding(), 
        this.getStride(), 
        this.getActivationFunctionName()
    );
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    super.initialization();
   
    if(padding > 0)
      throw new SInitializationException("Erreur SConvolutionLayerRW 003 : Un padding suprieur  0 n'est pas encore support.");
    
    // Vrifier qu'il y a au moins un filtre.
    if(this.filters_list.size() < 1)
      throw new SInitializationException("Erreur SConvolutionLayerRW 003 : La couche ne contient pas de filter.");
    
    // Vrifier que tous les filtres sont de mme taille (le input depth).
    int input_depth = this.filters_list.get(0).getInputDepth();
    
    for(int i = 1; i < this.filters_list.size(); i++)
      if(this.filters_list.get(i).getInputDepth() != input_depth)
        throw new SInitializationException("Erreur SConvolutionLayerRW 004 : Le filtre #" + (i+1) + " ne contient pas le mme nombre de matrice que le 1er filtre tait de " + input_depth + " filtres");
    
    // Vrifier que le nombre de biais concorde avec le nobmre de filtre.
    if(this.bias.length != this.filters_list.size())
      throw new SInitializationException("Erreur SConvolutionLayerRW 005 : Il y a " + this.filters_list.size() + " filtres et un nombre de biais de " + this.bias.length + " ce qui n'est pas gal.");
  
    // Validation des paramtres d'activation, s'il y en a.
    int outputs_size = SNeuralMath.computeOutputsSizeForSquareInputsDepth(input_size, filters_list.get(0).getInputDepth(), filters_list.size(), filters_list.get(0).getKernelOffset(), padding, stride);
    
    if(activation_parameters.getNbLine() != 0)
      if(activation_parameters.getMatrix().get(0).length != outputs_size)
        throw new SInitializationException("Erreur SConvolutionLayerRW 004 : La couche possde " + outputs_size + " neurones en sortie, mais contient " + activation_parameters.getMatrix().get(0).length + " lments par paramtre d'activation ce qui n'est pas gaux."); 
 
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_ACTIVATION_FUNCTION : this.activation_function = readIntOrExpression(remaining_line, SKeyWordDecoder.KW_ACTIVATION_FUNCTION, ACTIVATION_FUNCTIONS); return true;
      case SKeyWordDecoder.CODE_INPUT_SIZE : this.input_size = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_INPUT_SIZE); return true;
      case SKeyWordDecoder.CODE_STRIDE : this.stride = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_STRIDE); return true;
      case SKeyWordDecoder.CODE_PADDING : this.padding = this.readIntEqualOrGreaterThanZero(remaining_line, SKeyWordDecoder.KW_PADDING); return true;
     
      case SKeyWordDecoder.CODE_FILTERS : this.filters_list.add(new SFiltersRW(sbr)); return true;
      case SKeyWordDecoder.CODE_BIAS : this.bias = readDoubleArray(remaining_line, SKeyWordDecoder.KW_BIAS); return true;
      case SKeyWordDecoder.CODE_ACTIVATION_PARAMETERS : this.activation_parameters = new SMatrixRW(sbr); return true;
      
      default : return super.read(sbr, code, remaining_line);
    }
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
