package sim.readwrite.object.nn;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.exception.SConstructorException;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * ...
 * 
 * @author Simon Vezina
 * @since 2021-10-17
 * @version 2021-10-17
 */
public class SDropoutLayerRW extends SLayerRW {

  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = { SKeyWordDecoder.KW_DROPOUT_LAYER, SKeyWordDecoder.KW_INPUT_SIZE };
  
  /**
   * ...
   */
  private static final int DEFAULT_INPUT_SIZE = 1;
  
  /**
   * ...
   */
  private static final double DEFAULT_DROPOUT = 0.5;
  
  /**
   * ...
   */
  private int input_size;
  
  /**
   * ...
   */
  private double dropping;
  
  /**
   * ...
   */
  public SDropoutLayerRW() 
  {
    super();
    
    this.input_size = DEFAULT_INPUT_SIZE;
    this.dropping = DEFAULT_DROPOUT;
  }

  /**
   * ...
   * 
   * @param file_name
   */
  public SDropoutLayerRW(String file_name) 
  {
    super(file_name);
   
    this.input_size = DEFAULT_INPUT_SIZE;
    this.dropping = DEFAULT_DROPOUT;
  }

  /**
   * ...
   * 
   * @param file_name
   * @param input_size
   * @param dropout
   * @throws SConstructorException
   */
  public SDropoutLayerRW(String file_name, int input_size, double dropout) throws SConstructorException
  {
    super(file_name);
  
    this.dropping = dropout;
    this.input_size = input_size;
    
    try{
      initialization();
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SDropoutLayerRW 002 : Une erreur d'initialisation est survenue.", e);
    }
  }
    
  /**
   * ...
   * 
   * @param sbr
   * @throws IOException
   * @throws SConstructorException
   */
  public SDropoutLayerRW(SBufferedReader sbr) throws IOException, SConstructorException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SDropoutLayerRW 002 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  /**
   * Mthode pour obtenir le nombre de neurones en entre de la couche.
   * 
   * @return Le nombre de neurones en entre.
   */
  public int getInputSize()
  {
    return this.input_size;
  }
  
  /**
   * Mthode pour obtenir la probabilit du <i>drop</i> de la fonction dropout.
   *  
   * @return La probabilit du <i>drop</i>.
   */
  public double getDropout()
  {
    return this.dropping;
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_DROPOUT_LAYER;
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    // criture du la taille d'entre (input size).
    bw.write(SKeyWordDecoder.KW_INPUT_SIZE);
    bw.write("\t");
    bw.write(Integer.toString(this.input_size));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture de la probabilit du dropping.
    bw.write(SKeyWordDecoder.KW_DROPOUT);
    bw.write("\t\t");
    bw.write(Double.toString(this.dropping));
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_INPUT_SIZE : this.input_size = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_INPUT_SIZE); return true; 
      case SKeyWordDecoder.CODE_DROPOUT : this.dropping = this.readDoubleEqualOrGreaterThanValueAndEqualOrSmallerThanValue(remaining_line, 0.0, 1.0, SKeyWordDecoder.KW_DROPOUT); return true;
          
      default : return super.read(sbr, code, remaining_line);
    }
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    super.initialization();
   
    if(this.input_size < 1)
      throw new SInitializationException("Erreur SDropoutLayerRW 003 : Le nombre de neurone en entre est gal  " + this.input_size + " ce qui est infrieur  1.");
    
    if(this.dropping < 0.0 || this.dropping > 1.0)
      throw new SInitializationException("Erreur SDropoutLayerRW 003 : Le dropping est gal   " + this.dropping + " ce qui n'est pas entre 0 et 1.");
  }
}
