/**
 * 
 */
package sim.readwrite.object.nn;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.readwrite.SAbstractRW;
import sim.readwrite.object.SMatrixRW;
import sim.util.SBufferedReader;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SLayerRW</b> reprsente une couche de neurones abstraite pouvant tre lue dans un fichier texte.
 * 
 * @author Simon Vezina
 * @since 2021-09-17
 * @version 2022-02-28
 */
public class SLayerRW extends SAbstractRW {

  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    SKeyWordDecoder.KW_TRAINABLE 
  };
  
  /**
   * La constante <b>ACTIVATION_FUNCTIONS</b> reprsente la liste des fonctions d'activations pouvant tre reconnues.
   */
  public final static String[] ACTIVATION_FUNCTIONS = { "sigmoid", "relu", "softmax", "linear", "tanh", "gaussian", "softsign", "swish" };
  
  /**
   * La constante <b>AGGREGATION_FUNCTIONS</b> reprsente la liste des fonctions d'agrgation pouvant tre reconnues.
   */
  public final static String[] AGGREGATION_FUNCTIONS = { "weighted_sum", "quadric_sum" };
  
  /**
   * La constante <b>AGGREGATION_FUNCTIONS</b> reprsente la liste des fonctions d'agrgation pouvant tre reconnues.
   */
  public final static String[] POOLING_FUNCTIONS = { "max_pooling" };
  
  /**
   * ...
   */
  private final boolean DEFAULT_TRAINABLE = true;
  
  /**
   * ...
   */
  protected final static double[] DEFAULT_BIAS = { };
  
  /**
   * ...
   */
  protected final static SMatrixRW DEFAULT_MATRIX = new SMatrixRW();
  
  /**
   * ...
   */
  private boolean trainable;
  
  /**
   * ...
   */
  public SLayerRW()
  {
    super();
    
    this.trainable = DEFAULT_TRAINABLE;
  }
  
  /**
   * ....
   * 
   * @param file_name
   */
  public SLayerRW(String file_name)
  {
    super(file_name);
    
    this.trainable = DEFAULT_TRAINABLE;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public boolean getTrainable()
  {
    return this.trainable;
  }
  
  /**
   * ...
   * 
   * @param value
   */
  public void setTrainable(boolean value)
  {
    this.trainable = value;
  }
  
  @Override
  public String getRWName() 
  {
    throw new UnsupportedOperationException();
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
 
    bw.write(SKeyWordDecoder.KW_TRAINABLE);
    bw.write("\t\t");
    bw.write(Boolean.toString(this.trainable));
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_TRAINABLE : trainable = this.readTrueFalseExpressionOrInt(remaining_line, SKeyWordDecoder.KW_TRAINABLE); return true;  
      
      default : return super.read(sbr, code, remaining_line);
    }
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
