/**
 * 
 */
package sim.readwrite.object.nn;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Arrays;

import sim.exception.SConstructorException;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe </b>SPoolingLayerRW</b> reprsente la couche de neurones de type pooling pouvant tre lu dans un fichier.
 * 
 * @author Simon Vezina
 * @since 2021-10-19
 * @version 2022-02-28
 */
public class SPoolingLayerRW extends SLayerRW {

//--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = { SKeyWordDecoder.KW_INPUT_SIZE, 
      SKeyWordDecoder.KW_INPUT_DEPTH, SKeyWordDecoder.KW_FILTER_OFFSET, SKeyWordDecoder.KW_POOLING_FUNCTION
  };
  
  /**
   * ...
   */
  private static final int DEFAULT_INPUT_SIZE = 1;
  
  /**
   * ...
   */
  private static final int DEFAULT_INPUT_DEPTH = 1;
  
  /**
   * ...
   */
  private static final int DEFAULT_FILTER_OFFSET = 2;
  
  /**
   * ...
   */
  private int input_size;
  
  /**
   * ...
   */
  private int input_depth;
  
  /**
   * ...
   */
  private int filter_offset;
  
  /**
   * ...
   */
  private int pooling_function;
  
  /**
   * 
   */
  public SPoolingLayerRW()
  {
    super();
    
    this.input_size = DEFAULT_INPUT_SIZE;
    this.input_depth = DEFAULT_INPUT_DEPTH;
    this.filter_offset = DEFAULT_FILTER_OFFSET;
    this.pooling_function = 0;
  }

  /**
   * @param file_name
   */
  public SPoolingLayerRW(String file_name) 
  {
    super(file_name);
    
    this.input_size = DEFAULT_INPUT_SIZE;
    this.input_depth = DEFAULT_INPUT_DEPTH;
    this.filter_offset = DEFAULT_FILTER_OFFSET;
    this.pooling_function = 0;
  }
  
  /**
   * ...
   * 
   * @param inputs_size
   * @param inputs_depth
   * @param pooling_function_name
   */
  public SPoolingLayerRW(String file_name, int inputs_size, int inputs_depth, int filter_offset, String pooling_function_name)
  {
    super(file_name);    
    
    this.input_size = inputs_size;
    this.input_depth = inputs_depth;
    this.filter_offset = filter_offset;
    
    // Validation du nom de la fonction d'agrgation.
    int agg_index = Arrays.asList(POOLING_FUNCTIONS).indexOf(pooling_function_name);
    
    if(agg_index == -1)
      throw new SConstructorException("Erreur SPoolingLayerRW 001 : La fonction d'aggrgation '" + pooling_function_name + "' n'est pas reconnue. Les fonctions admissibles sont les suivantes : " + SStringUtil.join(POOLING_FUNCTIONS, ", "));
     
    try{
      initialization();
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SPoolingLayerRW 002 : Une erreur d'initialisation est survenue.", e);
    }
  }  

  /**
   * ...
   * 
   * @param sbr
   * @throws IOException
   * @throws SConstructorException
   */
  public SPoolingLayerRW(SBufferedReader sbr) throws IOException, SConstructorException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SConstructorException("Erreur SPoolingLayerRW 003 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  /**
   * Mthode pour obtenir le nombre de neurones en entre de la couche.
   * 
   * @return Le nombre de neurones en entre.
   */
  public int getInputSize()
  {
    return this.input_size;
  }
  
  /**
   * Mthode pour obtenir la profondeur d'entre de la couche.
   * 
   * @return La profondeur d'entre de la couche.
   */
  public int getInputDepth()
  {
    return this.input_depth;
  }
  
  /**
   * Mthode pour obtenir la taille du filtre carr de la fonction de regroupement.
   * 
   * @return La taille du filtre
   */
  public int getFilterOffset()
  {
    return this.filter_offset;
  }
  
  /**
   * Mthode pour obtenir le nom de la fonction de regroupement de la couche.
   * 
   * @return Le nom de la fonction de regroupement.
   */
  public String getPoolingFunctionName()
  {
    return POOLING_FUNCTIONS[pooling_function];
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_POOLING_LAYER;
  }
  
  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    // criture du la taille d'entre (input size).
    bw.write(SKeyWordDecoder.KW_INPUT_SIZE);
    bw.write("\t\t");
    bw.write(Integer.toString(this.input_size));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture de la profondeur
    bw.write(SKeyWordDecoder.KW_INPUT_DEPTH);
    bw.write("\t\t");
    bw.write(Integer.toString(this.input_depth));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture de la taille du filtre
    bw.write(SKeyWordDecoder.KW_FILTER_OFFSET);
    bw.write("\t\t");
    bw.write(Integer.toString(this.filter_offset));
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    // criture du nom de la fonction de pooling
    bw.write(SKeyWordDecoder.KW_POOLING_FUNCTION);
    bw.write("\t\t");
    bw.write(POOLING_FUNCTIONS[pooling_function]);
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_INPUT_SIZE : this.input_size = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_INPUT_SIZE); return true; 
      case SKeyWordDecoder.CODE_INPUT_DEPTH : this.input_depth = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_INPUT_DEPTH); return true; 
      case SKeyWordDecoder.CODE_FILTER_OFFSET : this.filter_offset = this.readIntEqualOrGreaterThanValue(remaining_line, DEFAULT_FILTER_OFFSET, SKeyWordDecoder.KW_FILTER_OFFSET); return true;
      case SKeyWordDecoder.CODE_POOLING_FUNCTION : this.pooling_function = readIntOrExpression(remaining_line, SKeyWordDecoder.KW_POOLING_FUNCTION, POOLING_FUNCTIONS); return true;
      
      default : return super.read(sbr, code, remaining_line);
    }
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    super.initialization();
   
    if(this.input_size < 1)
      throw new SInitializationException("Erreur SPoolingLayerRW 004 : Le nombre de neurone en entre est gal  " + this.input_size + " ce qui est infrieur  1.");
    
    if(this.input_depth < 1)
      throw new SInitializationException("Erreur SPoolingLayerRW 005 : La profondeur d'entre est gal  " + this.input_depth + " ce qui est infrieur  1.");
  
    if(this.filter_offset < DEFAULT_FILTER_OFFSET)
      throw new SInitializationException("Erreur SPoolingLayerRW 006 : Le filter est de taille " + this.filter_offset + " ce qui est infrieur  " + DEFAULT_FILTER_OFFSET + ".");
  }
  
}
