package sim.util;

import sim.exception.SRuntimeException;

/**
 * La classe SChronometer permet d'valuer le temps coul avec une rsolution en milliseconde.
 * @author Simon Vezina
 * @version 2014-12-08
 * @version 2021-08-22
 */
public class SChronometer 
{
	private long start_time;	// temps coul en milliseconde depuis January 1, 1970, 00:00:00 GMT pour signaler le dbut du chronomtre.
	private boolean isStarted;
	
	private long stop_time;		// temps coul en milliseconde depuis January 1, 1970, 00:00:00 GMT pour signaler l'arrt du chronomtre.
	private boolean isStoped;
	
	/**
	 * Constructeur du chronometre.
	 */
	public SChronometer()
	{
		reset();
	}
	
	/**
	 * Mthode pour initialiser le chronometre.
	 */
	public void start()
	{	
		start_time = System.currentTimeMillis();	// Temps  partir de January 1, 1970, 00:00:00 GMT
		isStarted = true;
	}
	
	/**
	 * Mthode pour arrter la mesure de l'coulement du temps depuis l'appel de la mthode start().
	 */
	public void stop()
	{
		stop_time = System.currentTimeMillis();		// Temps  partir de January 1, 1970, 00:00:00 GMT
		isStoped = true;
	}
	
	/**
	 * Mthode pour rinitialiser le chronomtre.
	 */
	public void reset()
	{
		start_time = 0;
		isStarted = false;
		
		stop_time = 0;
		isStoped = false;
	}
	
	/**
	 * Mthode pour dterminer si le chronomtre a t initialis.
	 * 
	 * @return <b>true</b> si le chronomtre a t intialis et <b>false</b> sinon.
	 */
	public boolean asStart()
	{
	  return isStarted;
	}
	
	/**
	 * Mthode pour obtenir l'coulement du temps entre l'appel de la fonction start() et stop()
	 * ou bien le temps coul entre l'appel de la fonction start() et cette fonction courante.
	 * 
	 * @return Temps coul en seconde.
	 * @throws SRuntimeException S'il l'appel de la mthode start() n'a pas encore initialise le chronomtre.
	 */
	public double getTime()throws SRuntimeException
	{
		return (double) getTimeMilliseconde() / 1000;
	}
	
	/**
	 * Mthode pour obtenir l'coulement du temps entre l'appel de la fonction start() et stop().
	 * ou bien le temps coul entre l'appel de la fonction start() et cette fonction courante.
	 * 
	 * @return Temps coul en milliseconde.
	 * @throws SRuntimeException S'il l'appel de la mthode start() n'a pas encore initialise le chronomtre.
	 */
	public long getTimeMilliseconde()throws SRuntimeException
	{
		if(!isStarted)
			throw new SRuntimeException("Erreur SChronometer 001 : La mthode start() du chronomtre n'a pas encore initialise le chronomtre.");
		
		if(isStoped)
			return stop_time - start_time;
		else
		{
			//Si le chronomtre n'a pas t stopp, retourner le temps coul entre le start() et l'appel de la fonction getTimeMilliseconde()
			long current_time = System.currentTimeMillis();
			return current_time - start_time;
		}
	}

  @Override
  public String toString()
  {
    return "SChronometer [TEMPS COUL : " + getTime() + " s ]";
  }
	
	
}
