/**
 * 
 */
package sim.math;

import java.util.ArrayList;
import java.util.List;

import sim.exception.SConstructorException;
import sim.exception.SNoImplementationException;

/**
 * <p>
 * La classe <b>SArrayMatrix</b> reprsente une matrice dont l'espace mmoire est exploit dans un tableau.
 * </p>
 * <p>
 * Ce type de matrice n'est pas bien adapt pour faire des oprations de ligne, mais permet une copie rapide de ses donnes.
 * </p>
 * 
 * @author Simon Vezina
 * @since 2019-04-13
 * @version 2022-09-13 (Version labo : Apprentissage par rseau de neurones)
 */
public class SArrayMatrix {

  //-------------
  // VARIABLES //
  //-------------
  
  /**
   * La variable <b>array</p> reprsente le tableau des donnes de la matrice. L'indice des donnes correspond  un parcours de la matrice ligne par ligne.
   */
  final double[] array;
  
  /**
   * La variable <b>offset</b> reprsente le nombre d'lment par ligne de la matrice.
   */
  final int offset;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur d'une matrice carr vide.
   * 
   * @param size Le nombre de donnes.
   * @throws SConstructorException Si la taille ne donne pas une matrice carre.
   */
  public SArrayMatrix(int size) throws SConstructorException
  {
    this(size, (int)Math.round(Math.sqrt((double)size)));
  }
  
  /**
   * <p>
   * Constructeur d'une matrice carr  partir de donnes.
   * </p>
   * <p>
   * Il est important de prciser que les donne seront recopies. Le tableau pass
   * en rfrence sera pas la rfrence des donnes de cette matrice.
   * </p>
   * 
   * @param array Le tableau des donnes (qui sera recopi)
   * 
   * @throws SConstructorException Si la taille ne donne pas une matrice carre.
   */
  public SArrayMatrix(double[] array) throws SConstructorException
  {
    this(array, (int)Math.round(Math.sqrt((double)array.length)));
  }
  /**
   * Constructeur d'une matrice vide en spcifiant le nombre de colonne.
   * 
   * @param size Le nombre d'lment dans la matrice.
   * @param offset Le nombre de colonne de la matrice (espacement entre chaque ligne).
   * @throws SConstructorException Si le nombre d'lment n'est pas en accord avec le nombre de colonne.
   */
  public SArrayMatrix(int size, int offset) throws SConstructorException
  {
    this(new double[size], offset);
  }
  
  /**
   * <p>
   * Constructeur d'une matrice  partir d'un tableau de valeurs.
   * </p>
   * <p>
   * Il est important de prciser que les donnes seront recopi. Ainsi, le tableau pass
   * en rfrence ne sera pas en rfrence comme tant les donnes de cette matrice.
   * </p>
   * 
   * @param array Le tableau des donnes (qui sera recopi).
   * @param offset Le nombre de colonnes.
   * @throws SConstructorException Si le nombre d'lments dans le tableau n'est pas en accord avec le nombre de colonne.
   */
  public SArrayMatrix(double[] array, int offset) throws SConstructorException
  {
    if(array.length < 1)
      throw new SConstructorException("Erreur SArrayMatrix 001 : La taille de la matrice doit tre suprieure  0.");
    
    if(offset < 1)
      throw new SConstructorException("Erreur SArrayMatrix 002 : Le offset de la matrice doit tre suprieur  0.");
    
    if((array.length / offset)*offset != array.length)
      throw new SConstructorException("Erreur SArrayMatrix 003 : La taille de la matrice size = " + array.length + " n'est pas conforme avec le offset = " + offset + ", car size / offset = " + (double)array.length/(double)offset + ".");
    
    this.offset = offset;
  
    // Faire une copie des donnes.
    this.array = array.clone();     
  }
  
  //------------
  // MTHODES //
  //------------
  
  /**
   * ...
   * 
   * @return
   */
  public int lenght()
  {
    return this.array.length;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int offset()
  {
    return this.offset;
  }
  
  /**
   * Mthode pour obtenir l'indice dans le tableau de la matrice  partir d'un positionnemt (ligne, colonne).
   * 
   * @param i L'indice de la ligne.
   * @param j L'indice de la colonne.
   * @return L'indice dans le tableau de la coordonne (i,j).
   */
  public int getArrayIndex(int i, int j)
  {
    return i*offset + j;
  }
  
  /**
   * ...
   * 
   * @param index
   * @return
   * @throws ArrayIndexOutOfBoundsException
   */
  public double get(int index) throws ArrayIndexOutOfBoundsException
  {
    return this.array[index];
  }
  
  /**
   * ...
   * 
   * @param i
   * @param j
   * @return
   * @throws ArrayIndexOutOfBoundsException
   */
  public double get(int i, int j) throws ArrayIndexOutOfBoundsException
  {
    return get(getArrayIndex(i,j));
  }
  
  /**
   * Mthode pour obtenir une valeur de la matrice en ligne i et colonne j.
   * Si l'lment (i,j) est  l'extrieur de la matrice, une valeur de 0.0 sera retourn
   * correspondant  une plage externe  la matrice de valeur nulle.
   * 
   * @param i L'indice de la ligne.
   * @param j L'indice de la colonne.
   * @return La valeur de la matrice en (i,j) incluant un <i>padding</i>.
   */
  public double getWithPadding(int i, int j)
  {
    if(i < 0 || j < 0)
      return 0.0;
    
    if(i > (nbLines()-1) || j > (nbColumns()-1))
      return 0.0;
    
    return get(i, j);
  }
  
  /**
   * ...
   * 
   * @param index
   * @param value
   * @throws ArrayIndexOutOfBoundsException
   */
  public void set(int index, double value) throws ArrayIndexOutOfBoundsException
  {
    this.array[index] = value;
  }
  
  /**
   * ...
   * 
   * @param i
   * @param j
   * @param value
   * @throws ArrayIndexOutOfBoundsException
   */
  public void set(int i, int j, double value) throws ArrayIndexOutOfBoundsException
  {
    this.array[getArrayIndex(i,j)] = value;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int nbColumns()
  {
    return this.offset;
  }
  
  /**
   * Mthode pour obtenir le nombre de lignes dans la matrice.
   * 
   * @return Le nombre de ligne dans la matrice.
   */
  public int nbLines()
  {
    return array.length / offset;
  }
  
  /**
   * ...
   * 
   * @param i
   * @param j
   * @param shift
   * @param m
   * @return
   */
  public double convolutionProduct(int i, int j, int shift, SArrayMatrix m)
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * ...
   * 
   * @param shift
   * @return
   */
  public SArrayMatrix convolutionProduct(int padding, int stride, SArrayMatrix m)
  {
    throw new SNoImplementationException();
    
  }
  
  /**
   * ...
   * 
   * @param index
   * @param m
   * @return
   */
  public double convolutionProduct(int index, SArrayMatrix m)
  {
    throw new SNoImplementationException();
  }
  
  /**
   * Mthode pour convertir un SArrayMatrix en SBufferedMatrix.
   * 
   * @return Une matrice en format buffer.
   */
  public SBufferedMatrix toSBufferedMatrix()
  {
    List<double[]> array_list = new ArrayList<double[]>();
    
    for(int i = 0; i < this.nbLines(); i++)
    {
      array_list.add(new double[this.nbColumns()]);
      for(int j = 0; j < this.nbColumns(); j++)
        array_list.get(i)[j] = this.get(i, j);
    }
    
    return new SBufferedMatrix(array_list);
  } 
  
}
