/**
 * 
 */
package sim.nn.layer;

import sim.nn.SActivable;
import sim.nn.gradient.SGradientLayer;

/**
 * <p>
 * L'interface <b>SLayer</b> reprsentant une couche de neurones. 
 * </p>
 * <p>
 * Une couche de neurones pourra contenir une ou plusieurs fonctions neuronales avec ou sans paramtres.
 * </p>
 *  
 * @author Simon Vzina
 * @since 2019-02-04
 * @version 2022-01-05
 */
public interface SLayer extends SActivable {

  public static final String FULL_CONNECTED_LAYER = "full-connected layer";
  public static final String POOLING_LAYER = "pooling layer";
  public static final String DROPOUT_LAYER = "dropout layer";
  public static final String CONVOLUTION_LAYER = "convolution layer";
  
  public static final String BATCH_FULL_CONNECTED_LAYER = "batch full-connected layer";
  public static final String BATCH_POOLING_LAYER = "batch pooling layer";
  public static final String BATCH_DROPOUT_LAYER = "batch dropout layer";
  public static final String BATCH_CONVOLUTION_LAYER = "batch convolution layer";
  
  /**
   * Mthode pour obtenir le nom de la couche de neurones.
   * 
   * @return Le nom de la couche.
   */
  public String getLayerName();
  
  /**
   * <p>
   * Mthode pour obtenir la description de la couche de neurones.
   * </p>
   * <p>
   * Les informations que l'on peut y retrouver sont la dimension de la couche ainsi que 
   * la liste des fonctions dans la squence d'activation.
   * </p>
   * @return La description de la couche de neurones.
   */
  public String getDescription();
      
  /**
   * Mthode pour raliser l'apprentissage sur les paramtres.
   * 
   * @param gradient Le gradient de la fonction d'erreur associ aux paramtres.
   * @param learning_rate Le taux d'apprentissage.
   * @throws IllegalArgumentException Si le type de gradient n'est pas adquant pour les paramtres.
   */
  public default void learn(SGradientLayer gradient, double learning_rate) throws IllegalArgumentException
  {
    // dans le cas d'une couche sans paramtre.
  }
  
  /**
   * <p>
   * Mthode pour raliser une copie d'une couche de neurones tout en gardant accs sur les mmes paramtres.
   * </p>
   * <p>
   * Ce type de copie n'est pas quivalente  la mthode <u>clone()</u>, car mme si la copie de la couche 
   * est sur un espace mmoire diffrent, les paramtres des deux couches partageront les mmes espaces mmoires.
   * Ce type de copie permet alors au deux couche d'effectuer leurs activations sur leur propre espace mmoire
   * tout en exploitant le mme espace mmoire occup par les paramtres ce qui correspond  un conomie.
   * </p>
   * 
   * @return Une copie de la couche avec usage des mmes paramtres en mmoire.
   */
  public SLayer copyWithSameParameter();
    
}
